! This is the velocity update. It uses the Boris algorithm as described by e.g.
! Birdsall and Langdon.

subroutine AdvanceVelocity(particles,E,Bmodel,B0,B1,B2,r1,r2, &
     Lx_min,Ly_min,Lz_min,dxyz,dt,ve,&
     num_local,max_per_proc,Nspecies,Nx_local,Ny_local,Nz_local,species)

  use SpecificTypes
  implicit none

  interface
     real*8 function Bfield(r, r1, r2, B1, B2)
       real*8, intent(in) :: r, r1, r2, B1, B2
     end function Bfield
  end interface
  
! Parameters
  integer, intent(in) :: num_local, max_per_proc, Nspecies, &
       Nx_local, Ny_local, Nz_local, Bmodel
  real*8, intent(in) :: Lx_min, Ly_min, Lz_min, dxyz(3), dt, ve(4)

  real*8, intent(in) :: B0(4), B1, B2, r1, r2
  real*8, intent(in) :: E(3,Nx_local+2,Ny_local+2,Nz_local+2)
  type(particlearrays) particles
  type(particlespecies) species(Nspecies)


! Local variables
  real*8 :: Ep(3), Bp(3), vplus(3), vmin(3),&
       vmid(3), etadt2(Nspecies)
  integer :: ii, jj, kk, pp, index_local(3)
  real*8 :: rest(3), drest(3), E_eff(3,2,2,2),&
       F1, F2, F3, F4, F5, F6, F7, F8, dV, tB(3), tBms, sB(3), rho

  dV = dxyz(1)*dxyz(2)*dxyz(3)
  etadt2 = 0.5d0*dt*species%eta

  if ( Bmodel .eq. 1 ) then
     ! Bmodel=1 means the magnetic field is constant
     ! Loop over all particles
     do pp = num_local, 1, -1

        ! Start interpolation of fields to particle positions.
        ! Loop over x, y, and z directions to find the index corresponding
        ! to the lowest x, y, z, which the particle should have a force from.

        rest(1) = dmod( particles%coordinates(1,pp)-Lx_min+0.5d0*dxyz(1), &
             dxyz(1) )
        rest(2) = dmod( particles%coordinates(2,pp)-Ly_min+0.5d0*dxyz(2), &
             dxyz(2) )
        rest(3) = dmod( particles%coordinates(3,pp)-Lz_min+0.5d0*dxyz(3), &
             dxyz(3) )
        drest   = dxyz-rest

        index_local(1)=1+idint( (particles%coordinates(1,pp)-Lx_min) / &
             dxyz(1)+0.5d0)
        index_local(2)=1+idint( (particles%coordinates(2,pp)-Ly_min) / &
             dxyz(2)+0.5d0)
        index_local(3)=1+idint( (particles%coordinates(3,pp)-Lz_min) / &
             dxyz(3)+0.5d0)


        ! Find fields Fx, Fy, Fz  at eight closest gridpoints
        do kk = 1, 2
           do jj = 1, 2
              do ii = 1, 2
                 E_eff(1,ii,jj,kk) = &
                      E(1,index_local(1)+ii-1,index_local(2)+jj-1, &
                      index_local(3)+kk-1)
                 E_eff(2,ii,jj,kk) = &
                      E(2,index_local(1)+ii-1,index_local(2)+jj-1, &
                      index_local(3)+kk-1)
                 E_eff(3,ii,jj,kk) = &
                      E(3,index_local(1)+ii-1,index_local(2)+jj-1, &
                      index_local(3)+kk-1)
              end do
           end do
        end do

        F1 = drest(1)*drest(2)*drest(3)
        F2 =  rest(1)*drest(2)*drest(3)
        F3 = drest(1)* rest(2)*drest(3)
        F4 = drest(1)*drest(2)* rest(3)
        F5 =  rest(1)* rest(2)*drest(3)
        F6 = drest(1)* rest(2)* rest(3)
        F7 =  rest(1)*drest(2)* rest(3)
        F8 =  rest(1)* rest(2)* rest(3)

        Ep(:) = E_eff(:,1,1,1)*F1+E_eff(:,2,1,1)*F2+ &
             E_eff(:,1,2,1)*F3+E_eff(:,1,1,2)*F4+ &
             E_eff(:,2,2,1)*F5+E_eff(:,1,2,2)*F6+ &
             E_eff(:,2,1,2)*F7+E_eff(:,2,2,2)*F8
        Ep(:) = Ep(:)/dV


        if ( species(particles%species(pp))%fluid ) then     ! This is Jesper's 
           if ( species(particles%species(pp))%gyration ) then ! fluid option
              Bp(1:3) = B0(1:3)

              ! Parallel to B
              vmin(1:3) = ( particles%coordinates(4,pp)*Bp(1) + &
                   particles%coordinates(5,pp)*Bp(2) + &
                   particles%coordinates(6,pp)*Bp(3) )*Bp(1:3)
              ! Adding E-cross-B drift
              vmid(1)   = vmin(1) + ( Ep(2)*Bp(3)-Ep(3)*Bp(2) )
              vmid(2)   = vmin(2) + ( Ep(3)*Bp(1)-Ep(1)*Bp(3) )
              vmid(3)   = vmin(3) + ( Ep(1)*Bp(2)-Ep(2)*Bp(1) )
              particles%coordinates(4:6,pp) = vmid(:)*sum( Bp(1:3)**2 )**(-1)
           end if
        else
           ! Normal calculation with Lorenz force
           ! The Boris scheme was described by Birdsall & Langdon.

!!!! Calculating 'tB' was done before

           ! Calculate vmin by moving to the frame where the ambient
           ! electric field is zero, and then add the first half acceleration
           vmin(:) = particles%coordinates(4:6,pp) - ve(1:3) + &
                etadt2( particles%species(pp) )*Ep(:)

           ! Cross product
           vmid(1) = vmin(1) +vmin(2)*species(particles%species(pp))%tB(3) - &
                vmin(3)*species(particles%species(pp))%tB(2)
           vmid(2) = vmin(2) +vmin(3)*species(particles%species(pp))%tB(1) - &
                vmin(1)*species(particles%species(pp))%tB(3)
           vmid(3) = vmin(3) +vmin(1)*species(particles%species(pp))%tB(2) - &
                vmin(2)*species(particles%species(pp))%tB(1)

!!!! Calculating 'sB' was done before

           ! Another cross product
           vplus(1) = vmin(1) +vmid(2)*species(particles%species(pp))%sB(3) - &
                vmid(3)*species(particles%species(pp))%sB(2)
           vplus(2) = vmin(2) +vmid(3)*species(particles%species(pp))%sB(1) - &
                vmid(1)*species(particles%species(pp))%sB(3)
           vplus(3) = vmin(3) +vmid(1)*species(particles%species(pp))%sB(2) - &
                vmid(2)*species(particles%species(pp))%sB(1)

           ! Set new velocities (second half acceleration),
           ! and add the ambient speed.
           particles%coordinates(4:6,pp) = vplus(:) + ve(1:3) + &
                etadt2( particles%species(pp) )*Ep(:)

        end if

     end do

  elseif ( Bmodel .eq. 2) then
     ! Bmodel=2 means By is given by the Bfield function and Bx=Bz=0
     ! Loop over all particles
     Bp = 0.0d0 ! we initialize with zeros. Bp(1) and Bp(3) will remain 0
     do pp = num_local, 1, -1

        ! Start interpolation of fields to particle positions.
        ! Loop over x, y, and z directions to find the index corresponding
        ! to the lowest x, y, z, which the particle should have a force from.

        rest(1) = dmod( particles%coordinates(1,pp)-Lx_min+0.5d0*dxyz(1), &
             dxyz(1) )
        rest(2) = dmod( particles%coordinates(2,pp)-Ly_min+0.5d0*dxyz(2), &
             dxyz(2) )
        rest(3) = dmod( particles%coordinates(3,pp)-Lz_min+0.5d0*dxyz(3), &
             dxyz(3) )
        drest   = dxyz-rest

        index_local(1)=1+idint( (particles%coordinates(1,pp)-Lx_min) / &
             dxyz(1)+0.5d0)
        index_local(2)=1+idint( (particles%coordinates(2,pp)-Ly_min) / &
             dxyz(2)+0.5d0)
        index_local(3)=1+idint( (particles%coordinates(3,pp)-Lz_min) / &
             dxyz(3)+0.5d0)


        ! Find fields Fx, Fy, Fz  at eight closest gridpoints
        do kk = 1, 2
           do jj = 1, 2
              do ii = 1, 2
                 E_eff(1,ii,jj,kk) = &
                      E(1,index_local(1)+ii-1,index_local(2)+jj-1, &
                      index_local(3)+kk-1)
                 E_eff(2,ii,jj,kk) = &
                      E(2,index_local(1)+ii-1,index_local(2)+jj-1, &
                      index_local(3)+kk-1)
                 E_eff(3,ii,jj,kk) = &
                      E(3,index_local(1)+ii-1,index_local(2)+jj-1, &
                      index_local(3)+kk-1)
              end do
           end do
        end do

        F1 = drest(1)*drest(2)*drest(3)
        F2 =  rest(1)*drest(2)*drest(3)
        F3 = drest(1)* rest(2)*drest(3)
        F4 = drest(1)*drest(2)* rest(3)
        F5 =  rest(1)* rest(2)*drest(3)
        F6 = drest(1)* rest(2)* rest(3)
        F7 =  rest(1)*drest(2)* rest(3)
        F8 =  rest(1)* rest(2)* rest(3)

        Ep(:) = E_eff(:,1,1,1)*F1+E_eff(:,2,1,1)*F2+ &
             E_eff(:,1,2,1)*F3+E_eff(:,1,1,2)*F4+ &
             E_eff(:,2,2,1)*F5+E_eff(:,1,2,2)*F6+ &
             E_eff(:,2,1,2)*F7+E_eff(:,2,2,2)*F8
        Ep(:) = Ep(:)/dV

        rho = sqrt( particles%coordinates(1,pp)**2.0d0 + &
             particles%coordinates(3,pp)**2.0d0 ) ! distance to y-axis
        Bp(2) = Bfield(rho, r1, r2, B1, B2) ! By component

        if ( species(particles%species(pp))%fluid ) then     ! This is Jesper's 
           if ( species(particles%species(pp))%gyration ) then ! fluid option

              ! Parallel to B
              vmin(1:3) = ( particles%coordinates(4,pp)*Bp(1) + &
                   particles%coordinates(5,pp)*Bp(2) + &
                   particles%coordinates(6,pp)*Bp(3) )*Bp(1:3)
              ! Adding E-cross-B drift
              vmid(1)   = vmin(1) + ( Ep(2)*Bp(3)-Ep(3)*Bp(2) )
              vmid(2)   = vmin(2) + ( Ep(3)*Bp(1)-Ep(1)*Bp(3) )
              vmid(3)   = vmin(3) + ( Ep(1)*Bp(2)-Ep(2)*Bp(1) )
              particles%coordinates(4:6,pp) = vmid(:)*sum( Bp(1:3)**2 )**(-1)
           end if
        else
           ! Normal calculation with Lorenz force
           ! The Boris scheme was described by Birdsall & Langdon.

           ! Calculating 'tB' 
           tB(1) = 0.5d0*dt*Bp(1)*species(particles%species(pp))%eta
           tB(2) = 0.5d0*dt*Bp(2)*species(particles%species(pp))%eta
           tB(3) = 0.5d0*dt*Bp(3)*species(particles%species(pp))%eta
           ! magnitude squared
           tBms = tB(1)**2.0d0 + tB(2)**2.0d0 + tB(3)**2.0d0
           
           ! Calculate vmin by moving to the frame where the ambient
           ! electric field is zero, and then add the first half acceleration
           vmin(:) = particles%coordinates(4:6,pp) - ve(1:3) + &
                etadt2( particles%species(pp) )*Ep(:)

           ! Cross product
           vmid(1) = vmin(1) +vmin(2)*tB(3) - vmin(3)*tB(2)
           vmid(2) = vmin(2) +vmin(3)*tB(1) - vmin(1)*tB(3)
           vmid(3) = vmin(3) +vmin(1)*tB(2) - vmin(2)*tB(1)

           ! Calculating 'sB'
           sB(1) = 2.0d0*tB(1)/(1.0d0 + tBms)
           sB(2) = 2.0d0*tB(2)/(1.0d0 + tBms)
           sB(3) = 2.0d0*tB(3)/(1.0d0 + tBms)
           
           ! Another cross product
           vplus(1) = vmin(1) +vmid(2)*sB(3) - vmid(3)*sB(2)
           vplus(2) = vmin(2) +vmid(3)*sB(1) - vmid(1)*sB(3)
           vplus(3) = vmin(3) +vmid(1)*sB(2) - vmid(2)*sB(1)

           ! Set new velocities (second half acceleration),
           ! and add the ambient speed.
           particles%coordinates(4:6,pp) = vplus(:) + ve(1:3) + &
                etadt2( particles%species(pp) )*Ep(:)

        end if

     end do

  end if

 
 return

end subroutine AdvanceVelocity

!------------------------------------------------------------------------


