recursive subroutine PrepareFiles(myid, Nspecies, Nprobes, probe, &
     attemptno, Nretries)

  use hdf5
  use mpi
  use ifport
  use SpecificTypes

  implicit none

  integer, intent(in) :: myid, Nretries, Nspecies, Nprobes
  integer, intent(inout) :: attemptno
  type(EandPprobe), intent(in) :: probe(Nprobes)

  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: plist_id      ! Property list identifier
  integer(hid_t) :: dset_id
  integer ierr
  type(C_PTR) :: f_ptr
  logical ex
  logical(4) result
  character filename*100, snumber*2,  pnumber*4, myidname*5
  character*11 groupname
  integer(hid_t) :: grp_id
  character*10 dsetname
  character*30 longname

  type probebuffer
     ! the radius of the probe for particle saving purposes
     real*8 rprobe
     ! real position of the probe
     real*8 rc(3)
  end type probebuffer
  ! The field that we wish to probe
  real*8 :: E(3,1)
  ! vector of the iterations at which we probe the field
  integer :: iterations(1)

  type(probebuffer) :: initialprobe(2)
  integer(hid_t) :: initialprobe_tid     ! File datatype identifier
  integer(hid_t) :: tid3      ! /* Nested Array Datatype ID	*/
  integer(hsize_t), dimension(2) :: vdims3=(/3, 1/)
  integer(hsize_t), dimension(1) :: vdims1=1
  integer, parameter :: LENGTH = 1
  integer, parameter :: rank = 1
  integer, parameter :: rankE = 2
  integer(hsize_t) :: DIM(1) = (/LENGTH/)   ! Dataspace dimensions
  integer(hsize_t), dimension(1:2) :: maxdims
  integer(hsize_t), dimension(1:2) :: dimsc = (/3,1/)
  integer(hsize_t), dimension(1) :: maxdims1
  integer(hsize_t), dimension(1) :: dimsc1 = 1
  integer(hsize_t), dimension(1:2) :: offset
  integer(hsize_t), dimension(1:2) :: count
  integer(hid_t) :: dataspace     ! Dataspace identifier
  integer(hid_t) :: memspace      ! Memory dataspace identifier
  integer(hid_t) :: crp_list      ! Dataset creation property identifier

  integer ii, ll
  
  attemptno = attemptno + 1

! process 0 creates directories whether they exist or not
  if (myid .eq. 0) then
     result = makedirqq('dumps')
     result = makedirqq('outp')
     result = makedirqq('outp/probes')
     result = makedirqq('outp/particles')
  end if

    call MPI_barrier(MPI_comm_world, ierr)
    if (ierr>0) then
       write (*,*) 'PrepareFiles, MPI_barrier, ierr=',ierr,' myid=',myid
    end if

! particles
  ! file name
  write(myidname,fmt='(I5.5)') myid
  filename = 'outp/particles/particles_'//'p'//myidname//'.h5'
  inquire(file=filename, exist=ex)
  if (.not. ex) then
     call h5fcreate_f(filename, h5f_acc_TRUNC_f, file_id, ierr)
     call h5fclose_f(file_id, ierr)
  end if

  ! B-field
  ! file name
  filename = 'outp/Bfield.h5'
  inquire(file=filename, exist=ex)
  if (ex) then
     ! If the file exists already, process 0 shall delete it
     if ( myid .eq. 0 ) then
        if (ierr == 0) then
           open(unit=1234, iostat=ierr, file=filename, status='old')
           close(1234, status='delete')
        end if
     end if
  end if
  call MPI_barrier(MPI_comm_world, ierr)
  if (ierr>0) then
     write (*,*) 'PrepareFiles, MPI_barrier, ierr=',ierr,' myid=',myid
  end if
  ! Setup file access property list with parallel I/O access.
  call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create the file collectively.
  call h5fcreate_f(filename, h5f_acc_trunc_f, file_id, ierr, &
       access_prp = plist_id)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! close property list and the file too
  call h5pclose_f(plist_id, ierr)
  if (ierr .ne. 0) then
     goto 98
  end if
  call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if

  ! E-field
  ! file name
  filename = 'outp/Efield.h5'
  inquire(file=filename, exist=ex)
  if (.not. ex) then
     ! Setup file access property list with parallel I/O access.
     call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 95
     end if
     call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)
     if (ierr .ne. 0) then
        goto 96
     end if
     ! Create the file collectively.
     call h5fcreate_f(filename, h5f_acc_trunc_f, file_id, ierr, &
          access_prp = plist_id)
     if (ierr .ne. 0) then
        goto 97
     end if
     ! close property list and the file too
     call h5pclose_f(plist_id, ierr)
     if (ierr .ne. 0) then
        goto 98
     end if
     call h5fclose_f(file_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
  end if

  ! potential
  ! file name
  filename = 'outp/potential.h5'
  inquire(file=filename, exist=ex)
  if (.not. ex) then
     ! Setup file access property list with parallel I/O access.
     call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 95
     end if
     call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)
     if (ierr .ne. 0) then
        goto 96
     end if
     ! Create the file collectively.
     call h5fcreate_f(filename, h5f_acc_trunc_f, file_id,ierr, &
          access_prp = plist_id)
     if (ierr .ne. 0) then
        goto 97
     end if
     ! close property list and the file too
     call h5pclose_f(plist_id, ierr)
     if (ierr .ne. 0) then
        goto 98
     end if
     call h5fclose_f(file_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
  end if

  ! density, one file per species
  ! file name
  do ll = 1, Nspecies
     write (snumber,fmt='(i2.2)') ll
     filename = 'outp/density'//snumber//'.h5'
     inquire(file=filename, exist=ex)
     if (.not. ex) then
        ! Setup file access property list with parallel I/O access.
        call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
        if (ierr .ne. 0) then
           goto 95
        end if
        call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)
        if (ierr .ne. 0) then
           goto 96
        end if
        ! Create the file collectively.
        call h5fcreate_f(filename, h5f_acc_trunc_f, file_id,ierr, &
             access_prp = plist_id)
        if (ierr .ne. 0) then
           goto 97
        end if
        ! close property list and the file too
        call h5pclose_f(plist_id, ierr)
        if (ierr .ne. 0) then
           goto 98
        end if
        call h5fclose_f(file_id, ierr)
        if (ierr .ne. 0) then
           goto 99
        end if
     end if
  end do

  ! flux, one file per species
  ! file name
  do ll = 1, Nspecies
     write (snumber,fmt='(i2.2)') ll
     filename = 'outp/flux'//snumber//'.h5'
     inquire(file=filename, exist=ex)
     if (.not. ex) then
        ! Setup file access property list with parallel I/O access.
        call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
        if (ierr .ne. 0) then
           goto 95
        end if
        call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)
        if (ierr .ne. 0) then
           goto 96
        end if
        ! Create the file collectively.
        call h5fcreate_f(filename, h5f_acc_trunc_f, file_id, ierr, &
             access_prp = plist_id)
        if (ierr .ne. 0) then
           goto 97
        end if
        ! close property list and the file too
        call h5pclose_f(plist_id, ierr)
        if (ierr .ne. 0) then
           goto 98
        end if
        call h5fclose_f(file_id, ierr)
        if (ierr .ne. 0) then
           goto 99
        end if
     end if
  end do

  ! Electric field probe
  ! The structure is set up here: one file per probe saved in the
  ! outp/probes directory. In this initialisation routine only
  ! timestep 0 is written, and for that timestep the field is set to 0.
  do ii = 1, Nprobes
     if (probe(ii)%mine) then
        ! Filename
        write (pnumber,fmt='(i4.4)') probe(ii)%probeno
        filename = 'outp/probes/Eprobe'//pnumber//'.h5'
        inquire(file=filename, exist=ex)
        if (.not. ex) then
           ! initialise the 'initialprobe' variable for use below
           initialprobe(1)%rprobe = probe(ii)%rprobe
           initialprobe(1)%rc = probe(ii)%rc
           initialprobe(2) = initialprobe(1) ! initialprobe(2) is a dummy
           E = 0.0d0
           iterations = 0
           ! Create the data space.
           call h5screate_simple_f(rank, dim, dataspace, ierr)
           if (ierr .ne. 0) then
              goto 84
           end if
           ! Create the file.
           call h5fcreate_f(filename, h5f_acc_TRUNC_f, file_id, ierr)
           if (ierr .ne. 0) then
              stop
              goto 97
           end if
           ! Create the memory data type.
           call h5tcreate_f(h5t_compound_f, &
                h5offsetof(C_LOC(initialprobe(1)), C_LOC(initialprobe(2))), &
                initialprobe_tid, ierr)
           if (ierr .ne. 0) then
              goto 85
           end if
           ! insert the various variables
           call h5tinsert_f(initialprobe_tid, "rprobe", &
                h5offsetof(C_LOC(initialprobe(1)), &
                C_LOC(initialprobe(1)%rprobe)), h5t_native_double, ierr)
           if (ierr .ne. 0) then
              goto 87
           end if
           call h5tarray_create_f(h5t_native_double, 1, vdims3, tid3, ierr)
           if (ierr .ne. 0) then
              goto 86
           end if
           call h5tinsert_f(initialprobe_tid, "rc", &
                h5offsetof(C_LOC(initialprobe(1)),C_LOC(initialprobe(1)%rc)), &
                tid3, ierr)
           if (ierr .ne. 0) then
              goto 87
           end if
           ! Create the dataset.
           dsetname = 'probe'
           call h5dcreate_f(file_id, dsetname, initialprobe_tid, dataspace, &
                dset_id, ierr)
           if (ierr .ne. 0) then
              goto 88
           end if
           ! Write data to the dataset
           f_ptr = C_LOC(initialprobe(1))
           call h5dwrite_f(dset_id, initialprobe_tid, f_ptr, ierr)
           if (ierr .ne. 0) then
              goto 89
           end if
           ! close everything except the file
           call h5dclose_f(dset_id, ierr)
           if (ierr .ne. 0) then
              goto 92
           end if
           call h5tclose_f(initialprobe_tid, ierr)
           if (ierr .ne. 0) then
              goto 90
           end if
           call h5sclose_f(dataspace, ierr)
           if (ierr .ne. 0) then
              goto 91
           end if

           !! Add extendable data sets for E and iterations
           ! E first
           ! data space with unlimited dimensions.
           dsetname = 'Efield'
           maxdims(1) = 3
           maxdims(2) = h5s_UNLIMITED_f
           call h5screate_simple_f(rankE, vdims3, dataspace, ierr, maxdims)
           if (ierr .ne. 0) then
              goto 84
           end if
           !Modify dataset creation properties, i.e. enable chunking
           call h5pcreate_f(h5p_DATASET_CREATE_f, crp_list, ierr)
           if (ierr .ne. 0) then
              goto 95
           end if
           call h5pset_chunk_f(crp_list, rankE, dimsc, ierr)
           if (ierr .ne. 0) then
              goto 93
           end if
           !Create a dataset
           call h5dcreate_f(file_id, dsetname, h5t_native_real, dataspace, &
                dset_id, ierr, crp_list )
           if (ierr .ne. 0) then
              goto 88
           end if
           !Write data array to dataset
           call h5dwrite_f(dset_id, h5t_native_real, real(E), vdims3, ierr)
           if (ierr .ne. 0) then
              goto 89
           end if
           !Close the objects that were opened.
           call h5dclose_f(dset_id, ierr)
           if (ierr .ne. 0) then
              goto 92
           end if
           call h5pclose_f(crp_list, ierr)
           if (ierr .ne. 0) then
              goto 98
           end if
           call h5sclose_f(dataspace, ierr)
           if (ierr .ne. 0) then
              goto 91
           end if
           ! Then iterations
           ! data space with unlimited dimensions.
           dsetname = 'iterations'
           maxdims1 = h5s_UNLIMITED_f
           call h5screate_simple_f(rank, vdims1, dataspace, ierr, maxdims1)
           if (ierr .ne. 0) then
              goto 84
           end if
           !Modify dataset creation properties, i.e. enable chunking
           call h5pcreate_f(h5p_DATASET_CREATE_f, crp_list, ierr)
           if (ierr .ne. 0) then
              goto 95
           end if
           call h5pset_chunk_f(crp_list, rank, dimsc1, ierr)
           if (ierr .ne. 0) then
              goto 93
           end if
           !Create a dataset
           call h5dcreate_f(file_id, dsetname, h5t_native_integer, dataspace, &
                dset_id, ierr, crp_list )
           if (ierr .ne. 0) then
              goto 88
           end if
           !Write data array to dataset
           call h5dwrite_f(dset_id, h5t_native_integer, iterations, vdims1,ierr)
           if (ierr .ne. 0) then
              goto 89
           end if
           !Close the objects that were opened.
           call h5dclose_f(dset_id, ierr)
           if (ierr .ne. 0) then
              goto 92
           end if
           call h5pclose_f(crp_list, ierr)
           if (ierr .ne. 0) then
              goto 98
           end if
           call h5sclose_f(dataspace, ierr)
           if (ierr .ne. 0) then
              goto 91
           end if

           ! close the file
           call h5fclose_f(file_id, ierr)
           if (ierr .ne. 0) then
              goto 99
           end if
        end if
     end if
  end do

  ! Particle probe
  ! The structure is set up here: one file per probe saved in the
  ! outp/probes directory. In this initialisation routine, the probe
  ! position and radius are written to the file. The actual saving of
  ! data will happen elsewhere.
  do ii = 1, Nprobes
     if (probe(ii)%mine) then
        ! Filename
        write (pnumber,fmt='(i4.4)') probe(ii)%probeno
        filename = 'outp/probes/Pprobe'//pnumber//'.h5'
        inquire(file=filename, exist=ex)
        if (.not. ex) then
           ! initialise the 'initialprobe' variable for use below
           initialprobe(1)%rprobe = probe(ii)%rprobe
           initialprobe(1)%rc = probe(ii)%rc
           initialprobe(2) = initialprobe(1) ! initialprobe(2) is a dummy
           ! Create the data space.
           call h5screate_simple_f(rank, dim, dataspace, ierr)
           if (ierr .ne. 0) then
              goto 84
           end if
           ! Create the file.
           call h5fcreate_f(filename, h5f_acc_TRUNC_f, file_id, ierr)
           if (ierr .ne. 0) then
              stop
              goto 97
           end if
           ! Create the memory data type.
           call h5tcreate_f(h5t_compound_f, &
                h5offsetof(C_LOC(initialprobe(1)), C_LOC(initialprobe(2))), &
                initialprobe_tid, ierr)
           if (ierr .ne. 0) then
              goto 85
           end if
           ! insert the various variables
           call h5tinsert_f(initialprobe_tid, "rprobe", &
                h5offsetof(C_LOC(initialprobe(1)), &
                C_LOC(initialprobe(1)%rprobe)), h5t_native_double, ierr)
           if (ierr .ne. 0) then
              goto 87
           end if
           call h5tarray_create_f(h5t_native_double, 1, vdims3, tid3, ierr)
           if (ierr .ne. 0) then
              goto 86
           end if
           call h5tinsert_f(initialprobe_tid, "rc", &
                h5offsetof(C_LOC(initialprobe(1)),C_LOC(initialprobe(1)%rc)), &
                tid3, ierr)
           if (ierr .ne. 0) then
              goto 87
           end if
           ! Create the dataset.
           dsetname = 'probe'
           call h5dcreate_f(file_id, dsetname, initialprobe_tid, dataspace, &
                dset_id, ierr)
           if (ierr .ne. 0) then
              goto 88
           end if
           ! Write data to the dataset
           f_ptr = C_LOC(initialprobe(1))
           call h5dwrite_f(dset_id, initialprobe_tid, f_ptr, ierr)
           if (ierr .ne. 0) then
              goto 89
           end if
           ! close everything 
           call h5dclose_f(dset_id, ierr)
           if (ierr .ne. 0) then
              goto 92
           end if
           call h5tclose_f(initialprobe_tid, ierr)
           if (ierr .ne. 0) then
              goto 90
           end if
           call h5sclose_f(dataspace, ierr)
           if (ierr .ne. 0) then
              goto 91
           end if
           call h5fclose_f(file_id, ierr)
           if (ierr .ne. 0) then
              goto 99
           end if
        end if
     end if
  end do

  
  return

! Error handling section
84 write (*,*) 'DumpFlux: error in the h5screate_simple_f call, myid=', myid
  goto 100
85 write (*,*) 'DumpFlux: error in the h5tcreate_f call, myid=', myid
  goto 100
86 write (*,*) 'DumpFlux: error in the h5tarray_create_f call, myid=', myid
  goto 100
87 write (*,*) 'DumpFlux: error in the h5tinsert_f call, myid=', myid
  goto 100
88 write (*,*) 'DumpFlux: error in the h5dcreate_f call, myid=', myid
  goto 100
89 write (*,*) 'DumpFlux: error in the h5dwrite_f call, myid=', myid
  goto 100
90 write (*,*) 'DumpFlux: error in the h5tclose_f call, myid=', myid
  goto 100
91 write (*,*) 'DumpFlux: error in the h5sclose_f call, myid=', myid
  goto 100
92 write (*,*) 'DumpFlux: error in the h5dclose_f call, myid=', myid
  goto 100
93 write (*,*) 'DumpFlux: error calling h5pset_chunk_f, myid=', myid
  goto 100
95 write (*,*) 'PrepareFiles: error calling h5pcreate_f, myid=', myid
  goto 100
96 write (*,*) 'PrepareFiles: error calling h5pset_fapl_mpio_f, myid=', myid
  goto 100
97 write (*,*) 'PrepareFiles: error calling h5fcreate_f, myid=', myid
  goto 100
98 write (*,*) 'PrepareFiles: error calling h5pclose_f, myid=', myid
  close(1,err=99)
  goto 100
99 write (*,*) 'PrepareFiles: error calling h5fclose_f, myid=', myid
100 if (attemptno<=Nretries) then
     call PrepareFiles(myid, Nspecies, Nprobes, probe, &
          attemptno, Nretries)
  end if

end subroutine PrepareFiles

!------------------------------------------------------------------------

recursive subroutine DumpParameters( Nspecies, species, attemptno, Nretries )

  use hdf5
  use SpecificTypes

  implicit none

  integer, intent(in) :: Nspecies, Nretries
    type(particlespecies), intent(in) ::  species(Nspecies)
  integer, intent(inout) :: attemptno

  integer(hid_t) :: spbuf_tid      ! File datatype identifier
  integer, parameter :: rank = 1
  integer(hsize_t) :: dim(1)         ! Dataspace dimensions
  integer(hid_t) :: dataspace        ! Dataspace identifier
  integer(hid_t) :: file_id          ! File identifier
  integer(hid_t) :: tid4             ! Nested Array Datatype ID
  integer(hsize_t), dimension(1) :: vdims4 = 4
  character*10 dsetname
  integer(hid_t) :: dset_id
    type(c_ptr) :: f_ptr
  integer ierr
  
  
  integer ii
  character indata*132, filename*242, snumber*2
  type(particlespecies) ::  spbuf(2)

  attemptno = attemptno + 1

  ! Assemble filename
  filename = 'outp/parameters.h5'
  ! Create the data space.
  dim = 1 ! size of the structured variable array species
  call h5screate_simple_f(rank, dim, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 84
  end if
  ! Create the file.
  call h5fcreate_f(filename, h5f_acc_TRUNC_f, file_id, ierr)
  if (ierr .ne. 0) then
     stop
     goto 84
  end if

  ! loop over the species number to create one dataset per species
  do ii = 1, Nspecies
     spbuf(1) = species(ii) ! spbuf(1) is the buffer we write from
     spbuf(2) = spbuf(1)    ! spbuf(2) is just a dummy variable
     ! Create the memory data type.
     call h5tcreate_f(h5t_compound_f, &
          h5offsetof(C_LOC(spbuf(1)), C_LOC(spbuf(2))), &
          spbuf_tid, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "gyration", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%gyration)), h5t_native_integer, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "fluid", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%fluid)), h5t_native_integer, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "cometion", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%cometion)), h5t_native_integer, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "mass", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%mass)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "charge", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%charge)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "upstreamdensity", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%upstreamdensity)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "upstreamkelvin", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%upstreamkelvin)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tarray_create_f(h5t_native_double, 1, vdims4, tid4, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "v0", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%v0)), tid4, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "vn", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%vn)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "Qn", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%Qn)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "Ek", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%Ek)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "nu_i", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%nu_i)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "n2p", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%n2p)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "tB", &
          h5offsetof(C_LOC(spbuf(1)),C_LOC(spbuf(1)%tB)), &
          tid4, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "sB", &
          h5offsetof(C_LOC(spbuf(1)),C_LOC(spbuf(1)%sB)), &
          tid4, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "eta", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%eta)), h5t_native_double, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "ipp", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%ipp)), h5t_native_integer, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "epp", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%epp)), h5t_native_integer, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5tinsert_f(spbuf_tid, "hpp", &
          h5offsetof(C_LOC(spbuf(1)), &
          C_LOC(spbuf(1)%hpp)), h5t_native_integer, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     ! Create the dataset.
     write (snumber,fmt='(i2.2)') ii
     dsetname = 'species'//snumber
     call h5dcreate_f(file_id, dsetname, spbuf_tid, dataspace, &
          dset_id, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     ! Write data to the dataset
     f_ptr = C_LOC(spbuf(1))
     call h5dwrite_f(dset_id, spbuf_tid, f_ptr, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     ! close things opened in the loop
     call h5tclose_f(spbuf_tid, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
     call h5dclose_f(dset_id, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
  end do

  ! close the dataspace and file
    call h5sclose_f(dataspace, ierr)
     if (ierr .ne. 0) then
        goto 84
     end if
   call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 84
  end if

  
  return

  ! Error handling section
84 write (*,*) 'DumpParameters: error calling some subroutine'
  goto 100
100 if (attemptno<=Nretries) then
     call DumpParameters( Nspecies, species, attemptno, Nretries)
  end if

end subroutine DumpParameters

!------------------------------------------------------------------------

recursive subroutine  DumpDump(Nspecies, num_local, max_per_proc, &
     particles, myid, iteration, U, Nx_local, Ny_local, Nz_local, &
     attemptno, Nretries)

  use SpecificTypes
  use mpi
  use ifport

  implicit none

  integer, intent(in) :: Nspecies, max_per_proc, iteration, myid, Nretries, &
       num_local, Nx_local, Ny_local, Nz_local
  integer, intent(inout) :: attemptno
  real*8, intent(in) :: U(Nx_local+2,Ny_local+2,Nz_local+2)
  type(particlearrays) particles

  integer ierr, ii, jj
  character filename*43, tmpfile*45, pnumber*5

  attemptno = attemptno + 1

! Everybody writes the particles
  write(pnumber,fmt='(i5.5)') myid
  tmpfile='dumps/XXdump_particles.p'//pnumber//'.picard.dump' 
  filename='dumps/dump_particles.p'//pnumber//'.picard.dump' 
  open(unit=1,form='unformatted',file=tmpfile,status='replace',err=97)
  write (1,err=98) num_local
  write (1,err=98) (particles%species(ii), ii=1, num_local)
  do jj = 1, num_local
     write (1,err=98) (particles%coordinates(ii,jj), ii=1,6)
  end do
  ! and the potential
  write (1,err=98) U
  close (1,err=99)

  ! Make sure all processes have written their temporary files 
  ! and then rename them.
  call MPI_Barrier(MPI_comm_world, ierr)
  if (ierr>0) then
     write (*,*) 'DumpDump, 1, ierr=',ierr,' myid=',myid
  end if
  ierr = rename(tmpfile,filename)
  if (ierr>0) then
     write (*,*) 'DumpDump, 2, ierr=',ierr,' myid=',myid
  end if

! Process 0 writes the iteration number
  if (myid==0) then
     tmpfile = 'dumps/XXdump_iteration.picard.dump'
     filename = 'dumps/dump_iteration.picard.dump'
     open(unit=1,file=tmpfile,status='replace',err=97)
     write (1,fmt='(I7)',err=98) iteration
     close(1,err=99)
     ierr = rename(tmpfile,filename)
     if (ierr>0) then
        write (*,*) 'DumpDump, 3, ierr=',ierr,' myid=',myid
     end if
  end if

  return

! Error handling section
97 write (*,*) 'DumpDump: error in open statement, myid=', myid
  goto 100
98 write (*,*) 'DumpDump: error in write statement, myid=', myid
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpDump: error in close statement, myid=', myid
100 if (attemptno<=Nretries) then
     call DumpDump(Nspecies, num_local, max_per_proc, &
          particles, myid, iteration, U, Nx_local, Ny_local, Nz_local, &
          attemptno, Nretries)
  end if

end subroutine DumpDump

!------------------------------------------------------------------------

recursive subroutine  LoadDump(Nspecies, num_local, max_per_proc, &
     particles, myid, iteration, U, Nx_local, Ny_local, Nz_local, &
     attemptno, Nretries)

  use SpecificTypes

  implicit none

  integer, intent(in) :: Nspecies, max_per_proc,  myid, Nretries
  integer, intent(inout) :: num_local, Nx_local, Ny_local, Nz_local, &
       iteration, attemptno
  real*8, intent(inout) :: U(Nx_local+2,Ny_local+2,Nz_local+2)
  type(particlearrays) particles

  integer ierr, ii, jj
  character filename*43, pnumber*5

  U = 0.0d0

  attemptno = attemptno + 1

  
! Read the iteration number
  filename = 'dumps/dump_iteration.picard.dump'
  open(unit=1,file=filename,status='old',err=97)
  read (1,fmt='(I7)',err=98) iteration
  close(1,err=99)


! Read the particles
  write(pnumber,fmt='(i5.5)') myid
  filename='dumps/dump_particles.p'//pnumber//'.picard.dump' 
  open(unit=1,form='unformatted',file=filename,status='old',err=97)
  read (1,err=98) num_local
  read (1,err=98) (particles%species(ii), ii=1, num_local)
  do jj = 1, num_local
     read (1,err=98) (particles%coordinates(ii,jj), ii=1,6)
  end do
  ! and the potential
  read (1,err=98) U  
  close (1,err=99)
  return

! Error handling section
97 write (*,*) 'LoadDump: error in open statement, myid=', myid
  goto 100
98 write (*,*) 'LoadDump: error in write statement, myid=', myid
  close(1,err=99)
  goto 100
99 write (*,*) 'LoadDump: error in close statement, myid=', myid
100 if (attemptno<=Nretries) then
     call LoadDump(Nspecies, num_local, max_per_proc, &
          particles, myid, iteration, U, Nx_local, Ny_local, Nz_local, &
          attemptno, Nretries)
  else
     write (*,*) 'LoadDump: disk access failed ',Nretries,' times, myid=', myid
     stop
  end if

end subroutine LoadDump

!------------------------------------------------------------------------

recursive subroutine DumpParticles( particles, num_local, iteration, &
     myid, attemptno, Nretries)

  use hdf5
  use SpecificTypes
  implicit none

  integer, intent(in) :: num_local, myid, Nretries
  type(particlearrays), intent(in) ::  particles
  integer, intent(inout) :: attemptno
  integer, intent(in) :: iteration

  character :: itername*7, myidname*5, filename*100
  real, allocatable ::  rbuf(:,:), vbuf(:,:)
  integer, allocatable ::  spbuf(:)
  integer ii, jj

  integer ierr
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer        ::   rank        ! Dataset rank
  character*11 groupname
  integer(hid_t) :: grp_id
  character*10 dsetname
  character*30 longname
  integer(hsize_t), dimension(1:2) :: dims
  integer(hsize_t), dimension(1)   :: dimsp
  integer(hid_t) :: dataspace     ! Dataspace identifier


  attemptno = attemptno + 1

  write(itername,fmt='(I7.7)') iteration
  write(myidname,fmt='(I5.5)') myid

  filename = 'outp/particles/particles_'//'p'//myidname//'.h5'

  ! allocate buffers
  allocate(rbuf(3,num_local))
  allocate(vbuf(3,num_local))
  allocate(spbuf(num_local))

  ! initialise buffers
  rbuf  = 0.0d0
  vbuf  = 0.0d0
  spbuf = 0
  ! put particles into the buffers
  do ii = 1, num_local
     rbuf(:,ii) = real(particles%coordinates(1:3,ii))
     vbuf(:,ii) = real(particles%coordinates(4:6,ii))
     spbuf(ii)  = particles%species(ii)
  end do

  !Open the file.
  call h5fopen_f(filename, h5f_acc_rdwr_f, file_id, ierr)
  if (ierr .ne. 0) then
     goto 91
  end if
  
  ! create a group for the current timestep
  groupname = 'iter'//itername
  call h5gcreate_f(file_id, groupname, grp_id, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if

  ! create and write to three datasets
  ! r:       particle positions
  ! v:       particle velocities
  ! species: partilce species

  ! --------------------------------- r --------------------------------- !
  ! Create the data space for the r dataset
  rank = 2
  dims(1) = 3
  dims(2) = num_local
  call h5screate_simple_f(rank, dims, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create the dataset with default properties.
  dsetname = 'r'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
       dset_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  ! Write the dataset.
  call h5dwrite_f(dset_id, h5t_native_real, &
       rbuf(:,1:num_local), dims, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if

  ! close the dataspace
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Close the dataset.
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if

    
  ! --------------------------------- v --------------------------------- !
  ! Create the data space for the v dataset
  rank = 2
  dims(1) = 3
  dims(2) = num_local
  call h5screate_simple_f(rank, dims, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create the dataset with default properties.
  dsetname = 'v'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
       dset_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  ! Write the dataset.
  call h5dwrite_f(dset_id, h5t_native_real, &
       vbuf(:,1:num_local), dims, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if

  ! close the dataspace
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Close the dataset.
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if


  ! ------------------------------ species ------------------------------ !
  ! Create the data space for the species dataset
  rank = 1
  dimsp = num_local
  call h5screate_simple_f(rank, dimsp, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create the dataset with default properties.
  dsetname = 'species'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_integer, dataspace, &
       dset_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  ! Write the dataset.
  call h5dwrite_f(dset_id, h5t_native_integer, &
       spbuf(1:num_local), dimsp, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if

  ! close the dataspace
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Close the dataset.
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if

  
  ! close the group
  call h5gclose_f(grp_id, ierr)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close the file.
  call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


  ! deallocate buffers
  deallocate(rbuf)
  deallocate(vbuf)
  deallocate(spbuf)

  return

  ! Error handling section
91 write (*,*) 'DumpPprobe: error calling h5fopen_f'
  goto 100
92 write (*,*) 'DumpPprobe: error calling h5gcreate_f'
  goto 100
93 write (*,*) 'DumpPprobe: error calling h5screate_simple_f'
  goto 100
94 write (*,*) 'DumpPprobe: error calling h5dcreate_f'
  goto 100
95 write (*,*) 'DumpPprobe: error calling h5dwrite_f'
  goto 100
96 write (*,*) 'DumpPprobe: error calling h5sclose_f'
  goto 100
97 write (*,*) 'DumpPprobe: error calling h5dclose_f'
  goto 100
98 write (*,*) 'DumpPprobe: error calling h5gclose_f'
  goto 100
99 write (*,*) 'DumpPprobe: error calling h5fclose_f'
100 if (attemptno<=Nretries) then
     call  DumpParticles( particles, num_local, iteration, myid, &
          attemptno, Nretries)
  end if

end subroutine DumpParticles

!------------------------------------------------------------------------

recursive subroutine DumpEfield( E, Nx_local, Ny_local, Nz_local, &
     iprocs, jprocs, kprocs, &
     iteration, Nspecies, myid, attemptno, Nretries)

  use hdf5
  use mpi
  
  implicit none

  integer, intent(in) :: Nx_local, Ny_local, Nz_local, &
       iprocs, jprocs, kprocs, Nspecies, myid
  real*8, intent(in) :: E(3,Nx_local+2, Ny_local+2, Nz_local+2)

  integer, intent(in) :: iteration, Nretries
  integer attemptno

  ! hdf5 handling variables
  character(len=2) :: dsetname    ! Dataset name
  character(len=100) :: filename  ! File name
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer(hid_t) :: dataspace     ! Dataspace identifier in file
  integer(hid_t) :: memspace      ! memspace identifier
  integer(hid_t) :: plist_id      ! Property list identifier
  integer :: rank = 3 ! Dataset rank
  integer(hsize_t), dimension(1:3) :: dimsf ! Dataset dimensions (this process)
  integer(hsize_t), dimension(1:3) :: dimsg ! Dimensions of the global variables
  integer(hsize_t), dimension(1:3) :: offset
  character*11 groupname
  integer(hid_t) :: grp_id
  character*19 longname

  ! locals
  integer :: ierr
  character :: itername*7

  
  attemptno = attemptno + 1
  
  write(itername,fmt='(I7.7)') iteration

  ! Write the E-field: Ex Ey Ez
  filename = 'outp/Efield.h5'

  ! Place in the file and size of block. Same for all of Ex, Ey, znd Ez
  dimsf(1)  = Nx_local
  dimsf(2)  = Ny_local
  dimsf(3)  = Nz_local
  dimsg(1)  = dimsf(1) * iprocs
  dimsg(2)  = dimsf(2) * jprocs
  dimsg(3)  = dimsf(3) * kprocs
  offset(1) = dimsf(1)*mod(myid+iprocs, iprocs)
  offset(2) = dimsf(2)*mod(myid/iprocs+jprocs, jprocs)
  offset(3) = dimsf(3)*mod(myid/(iprocs*jprocs)+kprocs, kprocs)
  
  ! setup property list
  call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 88
  end if
  call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)  
  if (ierr .ne. 0) then
     goto 89
  end if
  ! open the file
  call h5fopen_f(filename, h5f_acc_rdwr_f, file_id, ierr, plist_id)
  if (ierr .ne. 0) then
     goto 90
  end if

  ! create a group for the current timestep
  groupname = 'iter'//itername
  call h5gcreate_f(file_id, groupname, grp_id, ierr)
  if (ierr .ne. 0) then
     goto 91
  end if

! --------------------------------- Ex --------------------------------- !
  ! Create the data space for the Ex dataset.
  call h5screate_simple_f(rank, dimsg, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
  ! Create the dataset with default properties.
  dsetname = 'Ex'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
                      dset_id, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create property list for collective dataset write
  call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  ! Select the place we put the data in
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, dimsf, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create memory dataspace.
  call h5screate_simple_f(rank, dimsf, memspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! Write subset to dataset
  call h5dwrite_f(dset_id, h5t_native_real, &
       real(E(1,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)), dimsf, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close all Ex related resources
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5sclose_f(memspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


! --------------------------------- Ey --------------------------------- !
  ! Create the data space for the Ey dataset.
  call h5screate_simple_f(rank, dimsg, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
  ! Create the dataset with default properties.
  dsetname = 'Ey'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
                      dset_id, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create property list for collective dataset write
  call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  ! Select the place we put the data in
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, dimsf, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create memory dataspace.
  call h5screate_simple_f(rank, dimsf, memspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! Write subset to dataset
  call h5dwrite_f(dset_id, h5t_native_real, &
       real(E(2,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)), dimsf, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close all Ey related resources
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5sclose_f(memspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


! --------------------------------- Ez --------------------------------- !
  ! Create the data space for the Ez dataset.
  call h5screate_simple_f(rank, dimsg, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
  ! Create the dataset with default properties.
  dsetname = 'Ez'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
                      dset_id, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create property list for collective dataset write
  call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  ! Select the place we put the data in
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, dimsf, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create memory dataspace.
  call h5screate_simple_f(rank, dimsf, memspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! Write subset to dataset
  call h5dwrite_f(dset_id, h5t_native_real, &
       real(E(3,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)), dimsf, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close all Ez related resources
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5sclose_f(memspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


  ! Close all remaining resources.
  call h5gclose_f(grp_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5pclose_f(plist_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if

  return

  ! Error handling section
88 write (*,*) 'DumpEfield: error in the h5pcreate_f call'
  goto 100
89 write (*,*) 'DumpEfield: error in the h5pset_fapl_mpio_f call'
  goto 100
90 write (*,*) 'DumpEfield: error in the h5fopen_f call'
  goto 100
91 write (*,*) 'DumpEfield: error in the h5gcreate_f call'
  goto 100
92 write (*,*) 'DumpEfield: error in a h5screate_simple_f call'
  goto 100
93 write (*,*) 'DumpEfield: error in a h5dcreate_f call'
  goto 100
94 write (*,*) 'DumpEfield: error in a h5pcreate_f call'
  goto 100
95 write (*,*) 'DumpEfield: error in a h5pset_dxpl_mpio_f call'
  goto 100
96 write (*,*) 'DumpEfield: error in a h5sselect_hyperslab_f call'
  goto 100
97 write (*,*) 'DumpEfield: error in a h5screate_simple_f call'
  goto 100
98 write (*,*) 'DumpEfield: error in a h5dwrite_f call'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpEfield: error in a close statement'
100 if (attemptno<=Nretries) then
     call DumpEfield( E, Nx_local, Ny_local, Nz_local, &
          iprocs, jprocs, kprocs, &
          iteration, Nspecies, myid, attemptno, Nretries)
  end if

end subroutine DumpEfield

!------------------------------------------------------------------------

! This subroutine writes the magnetic field at the grid points.
recursive subroutine DumpBfield( Bmodel,B0,B1,B2,r1,r2, &
     Nx_local, Ny_local, Nz_local, &
     Lx_min,Lx_max,Ly_min,Ly_max,Lz_min,Lz_max,dxyz,&
     iprocs, jprocs, kprocs, &
     Nspecies, myid, attemptno, Nretries)

  use hdf5
  use mpi
  
  implicit none

  interface
     real*8 function Bfield(r, r1, r2, B1, B2)
       real*8, intent(in) :: r, r1, r2, B1, B2
     end function Bfield
  end interface

  integer, intent(in) :: Nx_local, Ny_local, Nz_local, &
       iprocs, jprocs, kprocs, Nspecies, myid, Bmodel
  real*8, intent(in) :: B0(4), B1, B2, r1, r2
  real*8, intent(in) :: Lx_min,Lx_max,Ly_min,Ly_max,Lz_min,Lz_max, dxyz(3)
  integer, intent(in) :: Nretries
  integer attemptno

  ! hdf5 handling variables
  character(len=2) :: dsetname    ! Dataset name
  character(len=100) :: filename  ! File name
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer(hid_t) :: dataspace     ! Dataspace identifier in file
  integer(hid_t) :: memspace      ! memspace identifier
  integer(hid_t) :: plist_id      ! Property list identifier
  integer :: rank = 3 ! Dataset rank
  integer(hsize_t), dimension(1:3) :: dimsf ! Dataset dimensions (this process)
  integer(hsize_t), dimension(1:3) :: dimsg ! Dimensions of the global variables
  integer(hsize_t), dimension(1:3) :: offset

  ! locals
  integer :: ierr, ii, jj, kk
  real*8 B(3,Nx_local+2, Ny_local+2, Nz_local+2), r

  ! find the B field at all points so that we can write it
  B = 0.0d0
  if ( Bmodel .eq. 1 ) then
     B(1,:,:,:) = B0(1)
     B(2,:,:,:) = B0(2)
     B(3,:,:,:) = B0(3)
  elseif ( Bmodel .eq. 2) then
     do kk = 1, Nz_local
        do jj = 1, Ny_local
           do ii = 1, Nx_local
              r = sqrt( (Lx_min+ii*dxyz(1))**2.0d0 + &
                   (Ly_min+jj*dxyz(2))**2.0d0 + &
                   (Lz_min+kk*dxyz(3))**2.0d0 )
              B(2,ii+1,jj+1,kk+1) = Bfield(r, r1, r2, B1, B2)
           end do
        end do
     end do
  end if

  attemptno = attemptno + 1

  ! Write the B-field: Bx By Bz
  filename = 'outp/Bfield.h5'

  ! Place in the file and size of block. Same for all of Bx, By, znd Bz
  dimsf(1)  = Nx_local
  dimsf(2)  = Ny_local
  dimsf(3)  = Nz_local
  dimsg(1)  = dimsf(1) * iprocs
  dimsg(2)  = dimsf(2) * jprocs
  dimsg(3)  = dimsf(3) * kprocs
  offset(1) = dimsf(1)*mod(myid+iprocs, iprocs)
  offset(2) = dimsf(2)*mod(myid/iprocs+jprocs, jprocs)
  offset(3) = dimsf(3)*mod(myid/(iprocs*jprocs)+kprocs, kprocs)

  ! setup property list
  call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 88
  end if
  call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)  
  if (ierr .ne. 0) then
     goto 89
  end if
  ! open the file
  call h5fopen_f(filename, h5f_acc_rdwr_f, file_id, ierr, plist_id)
  if (ierr .ne. 0) then
     goto 90
  end if

! --------------------------------- Bx --------------------------------- !
  ! Create the data space for the Bx dataset.
  call h5screate_simple_f(rank, dimsg, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
  ! Create the dataset with default properties.
  dsetname = 'Bx'
  call h5dcreate_f(file_id, dsetname, h5t_native_real, dataspace, &
                      dset_id, ierr)
  if (ierr .ne. 0) then
     ! If we end up here, the dataset probably exists already - try to open it!
     call h5dopen_f(file_id, dsetname, dset_id, ierr) 
     if (ierr .ne. 0) then
        goto 93
     end if
  end if
  ! Create property list for collective dataset write
  call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  ! Select the place we put the data in
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, dimsf, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create memory dataspace.
  call h5screate_simple_f(rank, dimsf, memspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! Write subset to dataset
  call h5dwrite_f(dset_id, h5t_native_real, &
       real(B(1,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)), dimsf, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close all Bx related resources
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5sclose_f(memspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


! --------------------------------- By --------------------------------- !
  ! Create the data space for the By dataset.
  call h5screate_simple_f(rank, dimsg, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
  ! Create the dataset with default properties.
  dsetname = 'By'
  call h5dcreate_f(file_id, dsetname, h5t_native_real, dataspace, &
                      dset_id, ierr)
  if (ierr .ne. 0) then
     ! If we end up here, the dataset probably exists already - try to open it!
     call h5dopen_f(file_id, dsetname, dset_id, ierr) 
     if (ierr .ne. 0) then
        goto 93
     end if
  end if
  ! Create property list for collective dataset write
  call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  ! Select the place we put the data in
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, dimsf, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create memory dataspace.
  call h5screate_simple_f(rank, dimsf, memspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! Write subset to dataset
  call h5dwrite_f(dset_id, h5t_native_real, &
       real(B(2,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)), dimsf, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close all By related resources
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5sclose_f(memspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


! --------------------------------- Bz --------------------------------- !
  ! Create the data space for the Bz dataset.
  call h5screate_simple_f(rank, dimsg, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
  ! Create the dataset with default properties.
  dsetname = 'Bz'
  call h5dcreate_f(file_id, dsetname, h5t_native_real, dataspace, &
                      dset_id, ierr)
  if (ierr .ne. 0) then
     ! If we end up here, the dataset probably exists already - try to open it!
     call h5dopen_f(file_id, dsetname, dset_id, ierr) 
     if (ierr .ne. 0) then
        goto 93
     end if
  end if
  ! Create property list for collective dataset write
  call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  ! Select the place we put the data in
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, dimsf, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create memory dataspace.
  call h5screate_simple_f(rank, dimsf, memspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! Write subset to dataset
  call h5dwrite_f(dset_id, h5t_native_real, &
       real(B(3,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)), dimsf, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close all Bz related resources
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5sclose_f(memspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


  ! Close all remaining resources.
  call h5pclose_f(plist_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if

  return

  ! Error handling section
88 write (*,*) 'DumpBfield: error in the h5pcreate_f call'
  goto 100
89 write (*,*) 'DumpBfield: error in the h5pset_fapl_mpio_f call'
  goto 100
90 write (*,*) 'DumpBfield: error in the h5fopen_f call'
  goto 100
91 write (*,*) 'DumpBfield: error in the h5gcreate_f call'
  goto 100
92 write (*,*) 'DumpBfield: error in a h5screate_simple_f call'
  goto 100
93 write (*,*) 'DumpBfield: error in a h5dcreate_f call'
  goto 100
94 write (*,*) 'DumpBfield: error in a h5pcreate_f call'
  goto 100
95 write (*,*) 'DumpBfield: error in a h5pset_dxpl_mpio_f call'
  goto 100
96 write (*,*) 'DumpBfield: error in a h5sselect_hyperslab_f call'
  goto 100
97 write (*,*) 'DumpBfield: error in a h5screate_simple_f call'
  goto 100
98 write (*,*) 'DumpBfield: error in a h5dwrite_f call'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpBfield: error in a close statement'
100 if (attemptno<=Nretries) then
     call DumpBfield( Bmodel,B0,B1,B2,r1,r2, Nx_local, Ny_local, Nz_local, &
          Lx_min,Lx_max,Ly_min,Ly_max,Lz_min,Lz_max,dxyz,&
          iprocs, jprocs, kprocs, &
          Nspecies, myid, attemptno, Nretries)
  end if

end subroutine DumpBfield

!------------------------------------------------------------------------

recursive subroutine DumpPotential(U, Nx_local, Ny_local, Nz_local, &
     iprocs, jprocs, kprocs, &
     iteration, Nspecies, myid, attemptno, Nretries)

  use hdf5
  use mpi
  
  implicit none

  integer, intent(in) :: Nx_local, Ny_local, Nz_local, &
       iprocs, jprocs, kprocs, Nspecies, myid
  real*8, intent(in) :: U(Nx_local+2, Ny_local+2, Nz_local+2)

  integer, intent(in) :: iteration, Nretries
  integer attemptno

  ! hdf5 handling variables
  character(len=9) :: dsetname    ! Dataset name
  character(len=100) :: filename  ! File name
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer(hid_t) :: dataspace     ! Dataspace identifier in file
  integer(hid_t) :: memspace      ! memspace identifier
  integer(hid_t) :: plist_id      ! Property list identifier
  integer :: rank = 3 ! Dataset rank
  integer(hsize_t), dimension(1:3) :: dimsf ! Dataset dimensions (this process)
  integer(hsize_t), dimension(1:3) :: dimsg ! Dimensions of the global variables
  integer(hsize_t), dimension(1:3) :: offset
  character*11 groupname
  integer(hid_t) :: grp_id
  character*30 longname

  ! locals
  integer :: ierr
  character :: itername*7

  
  attemptno = attemptno + 1
  
  write(itername,fmt='(I7.7)') iteration

  ! Write the potential
  filename = 'outp/potential.h5'

  ! Place in the file and size of block.
  dimsf(1)  = Nx_local
  dimsf(2)  = Ny_local
  dimsf(3)  = Nz_local
  dimsg(1)  = dimsf(1) * iprocs
  dimsg(2)  = dimsf(2) * jprocs
  dimsg(3)  = dimsf(3) * kprocs
  offset(1) = dimsf(1)*mod(myid+iprocs, iprocs)
  offset(2) = dimsf(2)*mod(myid/iprocs+jprocs, jprocs)
  offset(3) = dimsf(3)*mod(myid/(iprocs*jprocs)+kprocs, kprocs)
  
  ! setup property list
  call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 88
  end if
  call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)  
  if (ierr .ne. 0) then
     goto 89
  end if
  ! open the file
  call h5fopen_f(filename, h5f_acc_rdwr_f, file_id, ierr, plist_id)
  if (ierr .ne. 0) then
     goto 90
  end if

  ! create a group for the current timestep
  groupname = 'iter'//itername
  call h5gcreate_f(file_id, groupname, grp_id, ierr)
  if (ierr .ne. 0) then
     goto 91
  end if


  ! Create the data space for the potential dataset.
  call h5screate_simple_f(rank, dimsg, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
  ! Create the dataset with default properties.
  dsetname = 'potential'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
                      dset_id, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create property list for collective dataset write
  call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  ! Select the place we put the data in
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, dimsf, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Create memory dataspace.
  call h5screate_simple_f(rank, dimsf, memspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  ! Write subset to dataset
  call h5dwrite_f(dset_id, h5t_native_real, &
       real(U(2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)), dimsf, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 98
  end if
  
  ! Close all potential related resources
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5sclose_f(memspace, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


  ! Close all remaining resources.
  call h5gclose_f(grp_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5pclose_f(plist_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if
  call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if

  return

  ! Error handling section
88 write (*,*) 'DumpPotential: error in the h5pcreate_f call'
  goto 100
89 write (*,*) 'DumpPotential: error in the h5pset_fapl_mpio_f call'
  goto 100
90 write (*,*) 'DumpPotential: error in the h5fopen_f call'
  goto 100
91 write (*,*) 'DumpPotential: error in the h5gcreate_f call'
  goto 100
92 write (*,*) 'DumpPotential: error in a h5screate_simple_f call'
  goto 100
93 write (*,*) 'DumpPotential: error in a h5dcreate_f call'
  goto 100
94 write (*,*) 'DumpPotential: error in a h5pcreate_f call'
  goto 100
95 write (*,*) 'DumpPotential: error in a h5pset_dxpl_mpio_f call'
  goto 100
96 write (*,*) 'DumpPotential: error in a h5sselect_hyperslab_f call'
  goto 100
97 write (*,*) 'DumpPotential: error in a h5screate_simple_f call'
  goto 100
98 write (*,*) 'DumpPotential: error in a h5dwrite_f call'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpPotential: error in a close statement'
100 if (attemptno<=Nretries) then
     call DumpPotential(U, Nx_local, Ny_local, Nz_local, &
          iprocs, jprocs, kprocs, &
          iteration, Nspecies, myid, attemptno, Nretries)
  end if

end subroutine DumpPotential

!------------------------------------------------------------------------

recursive subroutine DumpDensity(N, Nx_local, Ny_local, Nz_local, &
     iprocs, jprocs, kprocs, &
     iteration, Nspecies, dV, myid, attemptno, Nretries)

  use hdf5
  use mpi
  
  implicit none

  integer, intent(in) :: Nx_local, Ny_local, Nz_local, &
       iprocs, jprocs, kprocs, Nspecies, myid
  real*8, intent(in) :: N(Nspecies, Nx_local+2, Ny_local+2, Nz_local+2)
  real*8, intent(in) :: dV

  integer, intent(in) :: iteration, Nretries
  integer attemptno

  ! hdf5 handling variables
  character(len=9) :: dsetname    ! Dataset name
  character(len=100) :: filename  ! File name
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer(hid_t) :: dataspace     ! Dataspace identifier in file
  integer(hid_t) :: memspace      ! memspace identifier
  integer(hid_t) :: plist_id      ! Property list identifier
  integer :: rank = 3 ! Dataset rank
  integer(hsize_t), dimension(1:3) :: dimsf ! Dataset dimensions (this process)
  integer(hsize_t), dimension(1:3) :: dimsg ! Dimensions of the global variables
  integer(hsize_t), dimension(1:3) :: offset
  character*11 groupname
  integer(hid_t) :: grp_id
  character*30 longname

  ! locals
  integer :: ierr, ll
  character :: itername*7, snumber*2

  
  attemptno = attemptno + 1
  
  write(itername,fmt='(I7.7)') iteration

  ! Place in the file and size of block.
  dimsf(1)  = Nx_local
  dimsf(2)  = Ny_local
  dimsf(3)  = Nz_local
  dimsg(1)  = dimsf(1) * iprocs
  dimsg(2)  = dimsf(2) * jprocs
  dimsg(3)  = dimsf(3) * kprocs
  offset(1) = dimsf(1)*mod(myid+iprocs, iprocs)
  offset(2) = dimsf(2)*mod(myid/iprocs+jprocs, jprocs)
  offset(3) = dimsf(3)*mod(myid/(iprocs*jprocs)+kprocs, kprocs)

  ! Write the density, one file per species
  do ll = 1, Nspecies
     write (snumber,fmt='(i2.2)') ll
     filename = 'outp/density'//snumber//'.h5'

     ! setup property list
     call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 88
     end if
     call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)  
     if (ierr .ne. 0) then
        goto 89
     end if
     ! open the file
     call h5fopen_f(filename, h5f_acc_rdwr_f, file_id, ierr, plist_id)
     if (ierr .ne. 0) then
        goto 90
     end if

     ! create a group for the current timestep
     groupname = 'iter'//itername
     call h5gcreate_f(file_id, groupname, grp_id, ierr)
     if (ierr .ne. 0) then
        goto 91
     end if


     ! Create the data space for the density dataset.
     call h5screate_simple_f(rank, dimsg, dataspace, ierr)
     if (ierr .ne. 0) then
        goto 92
     end if
     ! Create the dataset with default properties.
     dsetname = 'density'
     longname = groupname//'/'//dsetname
     call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
          dset_id, ierr)
     if (ierr .ne. 0) then
        goto 93
     end if
     ! Create property list for collective dataset write
     call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 94
     end if
     call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
     if (ierr .ne. 0) then
        goto 95
     end if
     ! Select the place we put the data in
     call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
          offset, dimsf, ierr)
     if (ierr .ne. 0) then
        goto 96
     end if
     ! Create memory dataspace.
     call h5screate_simple_f(rank, dimsf, memspace, ierr)
     if (ierr .ne. 0) then
        goto 97
     end if
     ! Write subset to dataset
     call h5dwrite_f(dset_id, h5t_native_real, &
          real(N(ll,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)/dV), dimsf, ierr, &
          memspace, dataspace)
     if (ierr .ne. 0) then
        goto 98
     end if

     ! Close all density related resources
     call h5sclose_f(dataspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5sclose_f(memspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5dclose_f(dset_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     ! Close all remaining resources.
     call h5gclose_f(grp_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5pclose_f(plist_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5fclose_f(file_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if

  end do

  return

  ! Error handling section
88 write (*,*) 'DumpDensity: error in the h5pcreate_f call'
  goto 100
89 write (*,*) 'DumpDensity: error in the h5pset_fapl_mpio_f call'
  goto 100
90 write (*,*) 'DumpDensity: error in the h5fopen_f call'
  goto 100
91 write (*,*) 'DumpDensity: error in the h5gcreate_f call'
  goto 100
92 write (*,*) 'DumpDensity: error in a h5screate_simple_f call'
  goto 100
93 write (*,*) 'DumpDensity: error in a h5dcreate_f call'
  goto 100
94 write (*,*) 'DumpDensity: error in a h5pcreate_f call'
  goto 100
95 write (*,*) 'DumpDensity: error in a h5pset_dxpl_mpio_f call'
  goto 100
96 write (*,*) 'DumpDensity: error in a h5sselect_hyperslab_f call'
  goto 100
97 write (*,*) 'DumpDensity: error in a h5screate_simple_f call'
  goto 100
98 write (*,*) 'DumpDensity: error in a h5dwrite_f call'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpDensity: error in a close statement'
100 if (attemptno<=Nretries) then
     call DumpDensity(N, Nx_local, Ny_local, Nz_local, &
          iprocs, jprocs, kprocs, &
          iteration, Nspecies, dV, myid, attemptno, Nretries)
  end if

end subroutine DumpDensity

!------------------------------------------------------------------------

recursive subroutine DumpFlux(F, Nx_local, Ny_local, Nz_local, &
     iprocs, jprocs, kprocs, &
     iteration, Nspecies, dV, myid, attemptno, Nretries)

  use hdf5
  use mpi
  
  implicit none

  integer, intent(in) :: Nx_local, Ny_local, Nz_local, &
       iprocs, jprocs, kprocs, Nspecies, myid
  real*8, intent(in) :: F(Nspecies,3,Nx_local+2, Ny_local+2, Nz_local+2)
  real*8, intent(in) :: dV

  integer, intent(in) :: iteration, Nretries
  integer attemptno

  ! hdf5 handling variables
  character(len=9) :: dsetname    ! Dataset name
  character(len=100) :: filename  ! File name
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer(hid_t) :: dataspace     ! Dataspace identifier in file
  integer(hid_t) :: memspace      ! memspace identifier
  integer(hid_t) :: plist_id      ! Property list identifier
  integer :: rank = 3 ! Dataset rank
  integer(hsize_t), dimension(1:3) :: dimsf ! Dataset dimensions (this process)
  integer(hsize_t), dimension(1:3) :: dimsg ! Dimensions of the global variables
  integer(hsize_t), dimension(1:3) :: offset
  character*11 groupname
  integer(hid_t) :: grp_id
  character*30 longname

  ! locals
  integer :: ierr, ll
  character :: itername*7, snumber*2

  
  attemptno = attemptno + 1
  
  write(itername,fmt='(I7.7)') iteration

  ! Place in the file and size of block.
  dimsf(1)  = Nx_local
  dimsf(2)  = Ny_local
  dimsf(3)  = Nz_local
  dimsg(1)  = dimsf(1) * iprocs
  dimsg(2)  = dimsf(2) * jprocs
  dimsg(3)  = dimsf(3) * kprocs
  offset(1) = dimsf(1)*mod(myid+iprocs, iprocs)
  offset(2) = dimsf(2)*mod(myid/iprocs+jprocs, jprocs)
  offset(3) = dimsf(3)*mod(myid/(iprocs*jprocs)+kprocs, kprocs)

  ! Write the flux, one file per species
  do ll = 1, Nspecies
     write (snumber,fmt='(i2.2)') ll
     filename = 'outp/flux'//snumber//'.h5'

     ! setup property list
     call h5pcreate_f(h5p_file_access_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 88
     end if
     call h5pset_fapl_mpio_f(plist_id, MPI_comm_world, MPI_info_null, ierr)  
     if (ierr .ne. 0) then
        goto 89
     end if
     ! open the file
     call h5fopen_f(filename, h5f_acc_rdwr_f, file_id, ierr, plist_id)
     if (ierr .ne. 0) then
        goto 90
     end if

     ! create a group for the current timestep
     groupname = 'iter'//itername
     call h5gcreate_f(file_id, groupname, grp_id, ierr)
     if (ierr .ne. 0) then
        goto 91
     end if


! --------------------------------- Fx --------------------------------- !
     ! Create the data space for the Fx dataset.
     call h5screate_simple_f(rank, dimsg, dataspace, ierr)
     if (ierr .ne. 0) then
        goto 92
     end if
     ! Create the dataset with default properties.
     dsetname = 'Fx'
     longname = groupname//'/'//dsetname
     call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
          dset_id, ierr)
     if (ierr .ne. 0) then
        goto 93
     end if
     ! Create property list for collective dataset write
     call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 94
     end if
     call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
     if (ierr .ne. 0) then
        goto 95
     end if
     ! Select the place we put the data in
     call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
          offset, dimsf, ierr)
     if (ierr .ne. 0) then
        goto 96
     end if
     ! Create memory dataspace.
     call h5screate_simple_f(rank, dimsf, memspace, ierr)
     if (ierr .ne. 0) then
        goto 97
     end if
     ! Write subset to dataset
     call h5dwrite_f(dset_id, h5t_native_real, &
          real(F(ll,1,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)/dV), dimsf, ierr, &
          memspace, dataspace)
     if (ierr .ne. 0) then
        goto 98
     end if

     ! Close all flux related resources
     call h5sclose_f(dataspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5sclose_f(memspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5dclose_f(dset_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if

! --------------------------------- Fy --------------------------------- !
     ! Create the data space for the Fy dataset.
     call h5screate_simple_f(rank, dimsg, dataspace, ierr)
     if (ierr .ne. 0) then
        goto 92
     end if
     ! Create the dataset with default properties.
     dsetname = 'Fy'
     longname = groupname//'/'//dsetname
     call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
          dset_id, ierr)
     if (ierr .ne. 0) then
        goto 93
     end if
     ! Create property list for collective dataset write
     call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 94
     end if
     call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
     if (ierr .ne. 0) then
        goto 95
     end if
     ! Select the place we put the data in
     call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
          offset, dimsf, ierr)
     if (ierr .ne. 0) then
        goto 96
     end if
     ! Create memory dataspace.
     call h5screate_simple_f(rank, dimsf, memspace, ierr)
     if (ierr .ne. 0) then
        goto 97
     end if
     ! Write subset to dataset
     call h5dwrite_f(dset_id, h5t_native_real, &
          real(F(ll,2,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)/dV), dimsf, ierr, &
          memspace, dataspace)
     if (ierr .ne. 0) then
        goto 98
     end if

     ! Close all flux related resources
     call h5sclose_f(dataspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5sclose_f(memspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5dclose_f(dset_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if

! --------------------------------- Fz --------------------------------- !
     ! Create the data space for the Fz dataset.
     call h5screate_simple_f(rank, dimsg, dataspace, ierr)
     if (ierr .ne. 0) then
        goto 92
     end if
     ! Create the dataset with default properties.
     dsetname = 'Fz'
     longname = groupname//'/'//dsetname
     call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
          dset_id, ierr)
     if (ierr .ne. 0) then
        goto 93
     end if
     ! Create property list for collective dataset write
     call h5pcreate_f(h5p_dataset_xfer_f, plist_id, ierr)
     if (ierr .ne. 0) then
        goto 94
     end if
     call h5pset_dxpl_mpio_f(plist_id, h5fD_mpio_collective_f, ierr)
     if (ierr .ne. 0) then
        goto 95
     end if
     ! Select the place we put the data in
     call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
          offset, dimsf, ierr)
     if (ierr .ne. 0) then
        goto 96
     end if
     ! Create memory dataspace.
     call h5screate_simple_f(rank, dimsf, memspace, ierr)
     if (ierr .ne. 0) then
        goto 97
     end if
     ! Write subset to dataset
     call h5dwrite_f(dset_id, h5t_native_real, &
          real(F(ll,3,2:Nx_local+1,2:Ny_local+1,2:Nz_local+1)/dV), dimsf, ierr, &
          memspace, dataspace)
     if (ierr .ne. 0) then
        goto 98
     end if

     ! Close all flux related resources
     call h5sclose_f(dataspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5sclose_f(memspace, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5dclose_f(dset_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if


     ! Close all remaining resources.
     call h5gclose_f(grp_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5pclose_f(plist_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if
     call h5fclose_f(file_id, ierr)
     if (ierr .ne. 0) then
        goto 99
     end if

  end do

  return

  ! Error handling section
88 write (*,*) 'DumpFlux: error in the h5pcreate_f call'
  goto 100
89 write (*,*) 'DumpFlux: error in the h5pset_fapl_mpio_f call'
  goto 100
90 write (*,*) 'DumpFlux: error in the h5fopen_f call'
  goto 100
91 write (*,*) 'DumpFlux: error in the h5gcreate_f call'
  goto 100
92 write (*,*) 'DumpFlux: error in a h5screate_simple_f call'
  goto 100
93 write (*,*) 'DumpFlux: error in a h5dcreate_f call'
  goto 100
94 write (*,*) 'DumpFlux: error in a h5pcreate_f call'
  goto 100
95 write (*,*) 'DumpFlux: error in a h5pset_dxpl_mpio_f call'
  goto 100
96 write (*,*) 'DumpFlux: error in a h5sselect_hyperslab_f call'
  goto 100
97 write (*,*) 'DumpFlux: error in a h5screate_simple_f call'
  goto 100
98 write (*,*) 'DumpFlux: error in a h5dwrite_f call'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpFlux: error in a close statement'
100 if (attemptno<=Nretries) then
     call DumpFlux(F, Nx_local, Ny_local, Nz_local, &
          iprocs, jprocs, kprocs, &
          iteration, Nspecies, dV, myid, attemptno, Nretries)
  end if

end subroutine DumpFlux

!------------------------------------------------------------------------

recursive subroutine DumpEprobe(Nprobes, thisprobe, fields_collected, &
     attemptno, Nretries)

  use hdf5
  use SpecificTypes

  implicit none

  integer Nprobes, fields_collected, attemptno, Nretries
  type(EandPprobe), intent(in) :: thisprobe
 
  integer :: rank
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer(hid_t) :: dataspace     ! Dataspace identifier
  integer(hid_t) :: memspace      ! Memory dataspace identifier
  integer(hsize_t), dimension(1:2) :: dims, maxdims
  integer(hsize_t), dimension(1:2) :: size
  integer(hsize_t), dimension(1:2) :: offset
  integer(hsize_t), dimension(1:2) :: count

  character filename*100, dsetname*10,  pnumber*4
  integer ii, ierr
  
  attemptno = attemptno + 1

  ! Assemble filename
  write (pnumber,fmt='(i4.4)') thisprobe%probeno
  filename = 'outp/probes/Eprobe'//pnumber//'.h5'
  !Open the file.
  call h5fopen_f (filename, h5f_acc_rdwr_f, file_id, ierr)
  if (ierr .ne. 0) then
     goto 87
  end if
  !Open the  dataset.
  dsetname = 'Efield'
  call h5dopen_f(file_id, dsetname, dset_id, ierr)
  if (ierr .ne. 0) then
     goto 88
  end if
  !Get dataset's dataspace handle.
  call h5dget_space_f(dset_id, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 89
  end if
  !Get dataspace's rank.
  call h5sget_simple_extent_ndims_f(dataspace, rank, ierr)
  if (ierr .ne. 0) then
     goto 90
  end if
  !Get dataspace's dimensions.
  call h5sget_simple_extent_dims_f(dataspace, dims, maxdims, ierr)
  if (ierr .lt. 0) then
     goto 91
  end if
  !Extend the dataset. 
  size(1)   = 3
  size(2)   = dims(2) + fields_collected
  call h5dset_extent_f(dset_id, size, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
   !create memory dataspace
  offset(1) = 0
  offset(2) = dims(2)
  count(1:2)  = (/3,fields_collected/)
  call h5screate_simple_f(rank, count, memspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  !Write to extended part of dataset
  call h5dget_space_f(dset_id, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset, count, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  call H5dwrite_f(dset_id, h5t_native_real,real(thisprobe%E), count, ierr, &
       memspace, dataspace)
  if (ierr .ne. 0) then
     goto 96
  end if

  ! Close the objects that were opened but not the file
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 98
  end if

  ! Do it all over for the iterations
  !
  !Open the  dataset.
  dsetname = 'iterations'
  call h5dopen_f(file_id, dsetname, dset_id, ierr)
  if (ierr .ne. 0) then
     goto 88
  end if
  !Get dataset's dataspace handle.
  call h5dget_space_f(dset_id, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 89
  end if
  !Get dataspace's rank.
  call h5sget_simple_extent_ndims_f(dataspace, rank, ierr)
  if (ierr .ne. 0) then
     goto 90
  end if
  !Get dataspace's dimensions.
  call h5sget_simple_extent_dims_f(dataspace, dims(1), maxdims(1), ierr)
  if (ierr .lt. 0) then
     goto 91
  end if
  !Extend the dataset. 
  size(1)   = dims(1) + fields_collected
  call h5dset_extent_f(dset_id, size(1), ierr)
  if (ierr .ne. 0) then
     goto 92
  end if
   !create memory dataspace
  offset(1) = dims(1)
  count(1)  = fields_collected
  call h5screate_simple_f(rank, count, memspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  !Write to extended part of dataset
  call h5dget_space_f(dset_id, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  call h5sselect_hyperslab_f(dataspace, h5s_select_set_f, &
       offset(1), count, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if
  call H5dwrite_f(dset_id, h5t_native_integer, thisprobe%iterations, &
       count, ierr, memspace, dataspace)
  if (ierr .ne. 0) then
     goto 96
  end if

  ! Close the objects that were opened
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 98
  end if

  ! close the file too
  call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if

  return

! Error handling section
87 write (*,*) 'DumpEprobe: error calling h5fopen_f'
  goto 100
88 write (*,*) 'DumpEprobe: error calling h5dopen_f'
  goto 100
89 write (*,*) 'DumpEprobe: error calling h5dget_space'
  goto 100
90 write (*,*) 'DumpEprobe: error calling h5sget_simple_extent_ndims_f'
  goto 100
91 write (*,*) 'DumpEprobe: error calling h5sget_simple_extent_dims_f'
  goto 100
92 write (*,*) 'DumpEprobe: error calling h5dset_extent_f'
  goto 100
93 write (*,*) 'DumpEprobe: error calling h5screate_simple_f'
  goto 100
94 write (*,*) 'DumpEprobe: error calling h5dget_space_f'
  goto 100
95 write (*,*) 'DumpEprobe: error calling h5sselect_hyperslab_f'
  goto 100
96 write (*,*) 'DumpEprobe: error calling H5dwrite_f'
  goto 100
97 write (*,*) 'DumpEprobe: error calling h5sclose_f'
  goto 100
98 write (*,*) 'DumpEprobe: error calling h5dclose_f'
  goto 100
99 write (*,*) 'DumpEprobe: error calling h5fclose_f'
100 stop
  if (attemptno<=Nretries) then
     call DumpEprobe(Nprobes, thisprobe, fields_collected, &
     attemptno, Nretries)
  end if

end subroutine DumpEprobe

!------------------------------------------------------------------------

recursive subroutine DumpPprobe(Nprobes, thisprobe, particles, num_local, &
     iteration, myid, attemptno, Nretries)

  use hdf5
  use SpecificTypes

  implicit none

  integer, intent(in) :: Nprobes, num_local, iteration, myid, Nretries
  integer, intent(inout) :: attemptno
  type(EandPprobe), intent(in) :: thisprobe
  type(particlearrays), intent(in) :: particles

  real*8 rp2
  character filename*69,  pnumber*4, itername*7
  integer ierr
  integer(hid_t) :: file_id       ! File identifier
  integer(hid_t) :: dset_id       ! Dataset identifier
  integer        ::   rank        ! Dataset rank
  character*11 groupname
  integer(hid_t) :: grp_id
  character*10 dsetname
  character*30 longname
  integer(hsize_t), dimension(1:2) :: dims
  integer(hsize_t), dimension(1)   :: dimsp
  integer(hid_t) :: dataspace     ! Dataspace identifier
  real, allocatable ::  rbuf(:,:), vbuf(:,:)
  integer, allocatable ::  spbuf(:)
  integer num_probe, ii

  attemptno = attemptno + 1

  ! allocate buffers
  allocate(rbuf(3,num_local))
  allocate(vbuf(3,num_local))
  allocate(spbuf(num_local))

  ! initialise buffers
  rbuf  = 0.0d0
  vbuf  = 0.0d0
  spbuf = 0
  ! put particles in the probe sphere into the buffers
  rp2 = thisprobe%rprobe**2.0d0
  num_probe = 0
  do ii = 1, num_local
     if (sum((particles%coordinates(1:3,ii) - &
          thisprobe%rc)**2.0d0) <= rp2) then
        num_probe = num_probe + 1
        rbuf(:,num_probe) = real(particles%coordinates(1:3,ii))
        vbuf(:,num_probe) = real(particles%coordinates(4:6,ii))
        spbuf(num_probe) = particles%species(ii)
     end if
  end do

  write(itername,fmt='(I7.7)') iteration
  ! Assemble filename
  write (pnumber,fmt='(i4.4)') thisprobe%probeno
  filename = 'outp/probes/Pprobe'//pnumber//'.h5'
  !Open the file.
  call h5fopen_f(filename, h5f_acc_rdwr_f, file_id, ierr)
  if (ierr .ne. 0) then
     goto 91
  end if
  
  ! create a group for the current timestep
  groupname = 'iter'//itername
  call h5gcreate_f(file_id, groupname, grp_id, ierr)
  if (ierr .ne. 0) then
     goto 92
  end if

  ! create and write to three datasets
  ! r:       particle positions
  ! v:       particle velocities
  ! species: partilce species

  ! --------------------------------- r --------------------------------- !
  ! Create the data space for the r dataset
  rank = 2
  dims(1) = 3
  dims(2) = num_probe
  call h5screate_simple_f(rank, dims, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create the dataset with default properties.
  dsetname = 'r'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
       dset_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  ! Write the dataset.
  call h5dwrite_f(dset_id, h5t_native_real, &
       rbuf(:,1:num_probe), dims, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if

  ! close the dataspace
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Close the dataset.
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if
  
  ! --------------------------------- v --------------------------------- !
  ! Create the data space for the v dataset
  rank = 2
  dims(1) = 3
  dims(2) = num_probe
  call h5screate_simple_f(rank, dims, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create the dataset with default properties.
  dsetname = 'v'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_real, dataspace, &
       dset_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  ! Write the dataset.
  call h5dwrite_f(dset_id, h5t_native_real, &
       vbuf(:,1:num_probe), dims, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if

  ! close the dataspace
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Close the dataset.
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if


  ! ------------------------------ species ------------------------------ !
  ! Create the data space for the species dataset
  rank = 1
  dimsp = num_probe
  call h5screate_simple_f(rank, dimsp, dataspace, ierr)
  if (ierr .ne. 0) then
     goto 93
  end if
  ! Create the dataset with default properties.
  dsetname = 'species'
  longname = groupname//'/'//dsetname
  call h5dcreate_f(file_id, longname, h5t_native_integer, dataspace, &
       dset_id, ierr)
  if (ierr .ne. 0) then
     goto 94
  end if
  ! Write the dataset.
  call h5dwrite_f(dset_id, h5t_native_integer, &
       spbuf(1:num_probe), dimsp, ierr)
  if (ierr .ne. 0) then
     goto 95
  end if

  ! close the dataspace
  call h5sclose_f(dataspace, ierr)
  if (ierr .ne. 0) then
     goto 96
  end if
  ! Close the dataset.
  call h5dclose_f(dset_id, ierr)
  if (ierr .ne. 0) then
     goto 97
  end if

  
  ! close the group
  call h5gclose_f(grp_id, ierr)
  if (ierr .ne. 0) then
     goto 98
  end if
  ! Close the file.
  call h5fclose_f(file_id, ierr)
  if (ierr .ne. 0) then
     goto 99
  end if


  ! deallocate buffers
  deallocate(rbuf)
  deallocate(vbuf)
  deallocate(spbuf)


  return

! Error handling section
91 write (*,*) 'DumpPprobe: error calling h5fopen_f'
  goto 100
92 write (*,*) 'DumpPprobe: error calling h5gcreate_f'
  goto 100
93 write (*,*) 'DumpPprobe: error calling h5screate_simple_f'
  goto 100
94 write (*,*) 'DumpPprobe: error calling h5dcreate_f'
  goto 100
95 write (*,*) 'DumpPprobe: error calling h5dwrite_f'
  goto 100
96 write (*,*) 'DumpPprobe: error calling h5sclose_f'
  goto 100
97 write (*,*) 'DumpPprobe: error calling h5dclose_f'
  goto 100
98 write (*,*) 'DumpPprobe: error calling h5gclose_f'
  goto 100
99 write (*,*) 'DumpPprobe: error calling h5fclose_f'
100 if (attemptno<=Nretries) then
     call DumpPprobe(Nprobes, thisprobe, particles, num_local, &
     iteration, myid, attemptno, Nretries)
  end if

end subroutine DumpPprobe

!------------------------------------------------------------------------
