! This program is intended to read the dump files in the dumps directory
! and to print some of the values in them, notably values of num_local.


program ExamineDumpFiles

  implicit none



  logical :: writeParticles, writeFields, xflow, yflow, zflow, &
       updateEfield, updateBfield, startfromdumpfile
  integer :: Nx, Ny, Nz, iteration, iter_end, iter_start, &
       ii, jj, kk, ss, num_local, num_global, maxit, &
       dump_period_dump, dump_period_particles, dump_period_pprobes, &
       dump_period_fields, dump_start, write_period_screen, &
       Nretries, attempts, Nprobes, &
       fields_collected
  real*8  :: Lx, Ly, Lz, Lx_min, Lx_max, Ly_min, Ly_max, Lz_min, Lz_max, & 
       Lx_local, Ly_local, Lz_local, xmin, xmax, ymin, ymax, &
       zmin, zmax, dxyz(3), &
       starttime, endtime, time, dt, maxerr, E0(4), B0(4), &
       uE_local, uE_global, uB_local, uB_global, uP_local, uP_global, &
       uEB0_global, uE0_global, uB0_global, uP0_global, u0_global, &
       dV, ve(4), nucleusradius, flatradius, Galandradius, fadeoutradius

 character filename*43, pnumber*5


! For MPI handling purposes
  integer :: myid, Nprocs, ierr


 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! 1.1 DOMAIN PARAMETERS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  ! should be multiple of 2
  integer iprocs, jprocs, kprocs
  ! should be equal to INTEGER*8-2, e.g., 6, 30, 54, 78, 102
  integer Nx_local, Ny_local, Nz_local
  ! number of species included in the simulation
  integer Nspecies
  ! should be a multiple of 8
  integer, parameter :: max_per_proc = 2**24


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! 1.3 Get parameters from file
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  ! Eventually all parameters should come from the file, but
  ! for now this is all that has been implemented
  call GetGeneralInput(iter_end, startfromdumpfile, dump_period_dump, &
       dump_period_particles, dump_period_pprobes, &
       dump_period_fields, dump_start, write_period_screen, &
       nucleusradius, flatradius, Galandradius, fadeoutradius, &
       iprocs, jprocs, kprocs, Nx_local, Ny_local, Nz_local, Nspecies, &
       xmin, xmax, ymin, ymax, zmin, zmax, dt, B0, Nprobes)

  Nprocs = iprocs * jprocs * kprocs

  ! Here's where we do the reading
  do ii=1,Nprocs
     write(pnumber,fmt='(i5.5)') ii-1
     filename='dumps/dump_particles.p'//pnumber//'.picard.dump' 
     open(unit=1,form='unformatted',file=filename,status='old',err=97)
     read (1,err=98) num_local
     close (1,err=99)
     write (*,*) 'process ', pnumber, ' num_local=',num_local
  end do
  goto 100

97 write (*,*) 'Error in open statement, myid=', myid
  goto 100
98 write (*,*) 'Error in write statement, myid=', myid
  close(1,err=99)
  goto 100
99 write (*,*) 'Error in close statement, myid=', myid
100 write (*,*) 'This is the end.'
  
end program ExamineDumpFiles


!------------------------------------------------------------------------

subroutine GetGeneralInput(iter_end, startfromdumpfile, dump_period_dump, &
     dump_period_particles, dump_period_pprobes, &
     dump_period_fields, dump_start, write_period_screen, &
     nucleusradius, flatradius, Galandradius, fadeoutradius, &
     iprocs, jprocs, kprocs, Nx_local, Ny_local, Nz_local, Nspecies, &
     xmin, xmax, ymin, ymax, zmin, zmax, dt, B0, Nprobes)

  implicit none

  logical startfromdumpfile
  integer iter_end, dump_period_dump, dump_period_particles, &
       dump_period_pprobes, dump_period_fields, dump_start, &
       write_period_screen, &
       iprocs, jprocs, kprocs, &
       Nx_local, Ny_local, Nz_local, Nspecies, Nprobes
  real*8  nucleusradius, flatradius, Galandradius, fadeoutradius, &
       xmin, xmax, ymin, ymax, zmin, zmax, dt, B0(4)
  
  integer i, j, k
  character indata*132, filename*242

  ! Defaults
  iter_end = 10000000  ! Bitter end
  startfromdumpfile = .false. ! start from dump or not
  dump_period_dump = 1000  ! iterations between dumps one can start from
  dump_period_particles=100000 ! number of iterations between particle dumps
  dump_period_pprobes=5000 ! number of iterations between particle probe dumps
  dump_period_fields=10   ! number of iterations between field dumps
  write_period_screen=10  ! number of iterations between showing onscreen
                          ! life signs.

  nucleusradius = 5.0d0;   ! radius of the nucleus [m], 2.0d3/400.0d0
  flatradius = 1.0d1       ! radius of flat density part [m], 4.0d3/400.0d0
  Galandradius = 8.0d2     ! outer radius of Galand model
  fadeoutradius = 8.8d2    ! radius at which the cometary ion density is zero
  iprocs = 2      ! processes in x dimension, should be multiple of 2
  jprocs = 2      ! processes in y dimension, should be multiple of 2
  kprocs = 2      ! processes in z dimension, should be multiple of 2
  Nx_local = 30   ! should be equal to INTEGER*8-2, e.g., 6, 30, 54, 78, 102
  Ny_local = 30   ! should be equal to INTEGER*8-2, e.g., 6, 30, 54, 78, 102
  Nz_local = 30   ! should be equal to INTEGER*8-2, e.g., 6, 30, 54, 78, 102
  Nspecies = 3    ! number of species included in the simulation
  xmin = -240.0d0  ! physical domain
  ymin = -240.0d0  ! physical domain
  zmin = -240.0d0  ! physical domain
  xmax =  240.0d0  ! physical domain
  ymax =  240.0d0  ! physical domain
  zmax =  240.0d0  ! physical domain
  dt   =  4.0d-7   ! timestep
  B0   = 0.0d0     ! Magnetic flux density
  Nprobes = 0      ! Number of electric field probes

  filename = 'inputpicarda1.m'
  open(unit=1,file=filename,status='old',err=98)
  goto 100
  
98 write (*,*) 'GetGeneralInput: Error opening input file ', filename
  stop
  
99 write (*,*) 'GetGeneralInput: Error reading input file ', filename
  stop
  
100 read (1,'(a)',err=99) indata
  do while (index(indata,'%END')+index(indata,'%end') == 0)
     i=index(indata,'=')
     if (i>0) then
        do j=1,i-1
           if(indata(j:j) /= ' ') exit
        enddo
        ! If there is a semicolon on the line, ignore it and everything beyond.
        k = index(indata(i+1:132),';')
        if (k==0) then
           k = 133-i
        end if
        select case (indata(j:i-1))
        case ('iter_end')
           read (indata(i+1:i+k-1),*) iter_end
        case ('startfromdumpfile')
           if (index(indata(i+1:i+k-1),'''yes''') > 0) then
              startfromdumpfile = .true.
           end if
        case ('dump_period_dump')
           read (indata(i+1:i+k-1),*) dump_period_dump
        case ('dump_period_particles')
           read (indata(i+1:i+k-1),*) dump_period_particles
        case ('dump_period_pprobes')
           read (indata(i+1:i+k-1),*) dump_period_pprobes
        case ('dump_period_fields')
           read (indata(i+1:i+k-1),*) dump_period_fields
        case ('write_period_screen')
           read (indata(i+1:i+k-1),*) write_period_screen
        case ('nucleusradius')
           read (indata(i+1:i+k-1),*) nucleusradius
        case ('flatradius')
           read (indata(i+1:i+k-1),*) flatradius
        case ('Galandradius', 'galandradius')
           read (indata(i+1:i+k-1),*) Galandradius
        case ('fadeoutradius')
           read (indata(i+1:i+k-1),*) fadeoutradius
        case ('iprocs')
           read (indata(i+1:i+k-1),*) iprocs
        case ('jprocs')
           read (indata(i+1:i+k-1),*) jprocs
        case ('kprocs')
           read (indata(i+1:i+k-1),*) kprocs
        case ('Nx_local', 'nx_local')
           read (indata(i+1:i+k-1),*) Nx_local
        case ('Ny_local', 'ny_local')
           read (indata(i+1:i+k-1),*) Ny_local
        case ('Nz_local', 'nz_local')
           read (indata(i+1:i+k-1),*) Nz_local
        case ('Nspecies', 'nspecies')
           read (indata(i+1:i+k-1),*) Nspecies
        case ('xmin')
           read (indata(i+1:i+k-1),*) xmin
        case ('ymin')
           read (indata(i+1:i+k-1),*) ymin
        case ('zmin')
           read (indata(i+1:i+k-1),*) zmin
        case ('xmax')
           read (indata(i+1:i+k-1),*) xmax
        case ('ymax')
           read (indata(i+1:i+k-1),*) ymax
        case ('zmax')
           read (indata(i+1:i+k-1),*) zmax
        case ('dt')
           read (indata(i+1:i+k-1),*) dt
        case ('B0x', 'b0x')
           read (indata(i+1:i+k-1),*) B0(1)
        case ('B0y', 'b0y')
           read (indata(i+1:i+k-1),*) B0(2)
        case ('B0z', 'b0z')
           read (indata(i+1:i+k-1),*) B0(3)
        case ('Nprobes', 'nprobes')
           read (indata(i+1:i+k-1),*) Nprobes
        case default
           if (index(indata(j:j+1),'%')==0) then
              write (*,*) 'Input quantity ', indata(j:i-1), ' is unknown.'
           end if
        end select
     end if
     read (1,'(a)') indata
  end do

  close(1,err=101)
  return

101 write (*,*) 'GetGeneralInput: Error closing input file ', filename
  stop

end subroutine GetGeneralInput

!------------------------------------------------------------------------
