! The ParticlesProduce subroutine produces particles in the interior of
! the simulation, i.e. in every cell that is not a guard cell. This 
! is meant to model ionisation of the neutrals that are present.
subroutine ParticlesProduce(particles, &
     Lx_min,Ly_min,Lz_min,dxyz,ve,&
     max_per_proc,num_local,Nspecies, &
     Nx_local,Ny_local,Nz_local,dt, &
     nucleusradius,flatradius,Galandradius,fadeoutradius,species)

  use SpecificTypes
  implicit none

  interface
     real*8 function Galand(nu, Q, r, nucleusradius, u)
       real*8, intent(in) :: nu, Q, r, nucleusradius, u
     end function Galand

     real*8 function Haser(nu, Q, r, nucleusradius, u)
       real*8, intent(in) :: nu, Q, r, nucleusradius, u
     end function Haser
  end interface

  ! Parameters
  integer, intent(in) :: max_per_proc, Nspecies, Nx_local, Ny_local, Nz_local
  real*8, intent(in) :: Lx_min, Ly_min, Lz_min, dxyz(3), ve(4), &
       dt, nucleusradius, flatradius,Galandradius,fadeoutradius

  integer, intent(inout) :: num_local
  type(particlearrays) particles
  type(particlespecies) species(Nspecies)

  real*8, parameter :: pi  = 3.141592653589793d0

  ! Local variables
  integer ee, pp, ss, ii, jj, kk, num_create
  real*8 rnd, rndp(7), x0, y0, z0, r1, nden, dV
  real*8, allocatable :: ui(:), ue(:), uh(:)

  allocate (ui(Nspecies) )
  allocate (ue(Nspecies) )
  allocate (uh(Nspecies) )
  dV = dxyz(1)*dxyz(2)*dxyz(3)
  ue = 0.0d0
  uh = 0.0d0
  ui = 0.0d0

  do ss = 1, Nspecies
     ue(ss)=dsqrt(2.0d0*species(ss)%mass*species(ss)%Ek* &
          (species(1)%mass*(species(ss)%mass+species(1)%mass))**(-1.0d0))
     uh(ss)=dsqrt(2.0d0*species(ss)%mass*species(ss)%Ek* &
          (species(2)%mass*(species(ss)%mass+species(2)%mass))**(-1.0d0))
     if (species(ss)%epp .gt. 0) then
        ui(ss) = dsqrt( 2.0d0*species(1)%mass*species(ss)%Ek* &
             (species(ss)%mass*(species(ss)%mass+species(1)%mass))**(-1.0d0) )
     else if (species(ss)%hpp .gt. 0) then
        ui(ss) = dsqrt( 2.0d0*species(2)%mass*species(ss)%Ek * &
             (species(ss)%mass*(species(ss)%mass+species(2)%mass))**(-1.0d0) )
     else
        ui(ss)  = 0.0d0
     end if
  end do
  
  ! Loop through each cell which is not a guard cell
  do kk = 2, Nz_local+1
     do jj = 2, Ny_local+1
        do ii = 2, Nx_local+1
           ! Lower edge of cells
           x0 = Lx_min + dxyz(1)*(ii-2)
           y0 = Ly_min + dxyz(2)*(jj-2)
           z0 = Lz_min + dxyz(3)*(kk-2)
           r1 =dsqrt((x0+0.5d0*dxyz(1))**2.0d0 + (y0+0.5d0*dxyz(2))**2.0d0 + &
                (z0+0.5d0*dxyz(3))**2.0d0)
           if ( (4.0d0*r1)**3.0d0 .lt. dV) then
              r1 = ( dsqrt((x0+0.25d0*dxyz(1))**2.0d0 + &
                   (y0+0.25d0*dxyz(2))**2.0d0 + &
                   (z0+0.25d0*dxyz(3))**2.0d0) &
                   + dsqrt((x0+0.25d0*dxyz(1))**2.0d0 + &
                   (y0+0.25d0*dxyz(2))**2.0d0 + &
                   (z0+0.75d0*dxyz(3))**2) &
                   + dsqrt((x0+0.25d0*dxyz(1))**2.0d0 + &
                   (y0+0.75d0*dxyz(2))**2.0d0 + &
                   (z0+0.25d0*dxyz(3))**2) + &
                   dsqrt((x0+0.75d0*dxyz(1))**2.0d0 + &
                   (y0+0.25d0*dxyz(2))**2.0d0 + &
                   (z0+0.25d0*dxyz(3))**2) + &
                   dsqrt((x0+0.25d0*dxyz(1))**2.0d0 + &
                   (y0+0.75d0*dxyz(2))**2.0d0 + &
                   (z0+0.75d0*dxyz(3))**2) + &
                   dsqrt((x0+0.75d0*dxyz(1))**2.0d0 + &
                   (y0+0.25d0*dxyz(2))**2.0d0 + &
                   (z0+0.75d0*dxyz(3))**2) + &
                   dsqrt((x0+0.75d0*dxyz(1))**2.0d0 + &
                   (y0+0.75d0*dxyz(2))**2.0d0 + &
                   (z0+0.25d0*dxyz(3))**2) + &
                   dsqrt((x0+0.75d0*dxyz(1))**2.0d0 + &
                   (y0+0.75d0*dxyz(2))**2.0d0 + &
                   (z0+0.75d0*dxyz(3))**2) )*0.125d0
           end if

           do ss = 1, Nspecies
              if ((species(ss)%cometion) .and. (species(ss)%vn .gt. 0.0d0)) then
                 ! Create a neutral density profile following an
                 ! approximate Haser model, except inside nucleusradius,
                 ! where it is flat, and outside galandradius, where
                 ! it fades to zero which it reaches at fadeoutradius
                if ( r1 > fadeoutradius ) then
                    nden = 0.0d0
                 elseif ( r1 > Galandradius ) then
                    nden = ( 1.0d0 - (r1-Galandradius) / &
                         (fadeoutradius-Galandradius) ) * &
                         Haser(species(ss)%nu_i, species(ss)%Qn, &
                         Galandradius, nucleusradius, species(ss)%vn)
                 elseif ( r1 > nucleusradius ) then
                    nden = Haser(species(ss)%nu_i, species(ss)%Qn, r1, &
                         nucleusradius, species(ss)%vn)
                 else
                    nden=Haser(species(ss)%nu_i,species(ss)%Qn, &
                         nucleusradius,nucleusradius,species(ss)%vn)
                 end if
              else
                 nden = 0.0d0
              end if

              if ( species(ss)%n2p .gt. 0.0d0 ) then
                 call random_number(rnd)
                 num_create = idint( species(ss)%nu_i * dt * &
                      nden * dV / species(ss)%n2p + rnd )
              else
                 num_create = 0
              end if
              
              if (num_local+(1+species(ss)%epp+species(ss)%hpp)*num_create > &
                   max_per_proc) then
                 write (*,*) 'Cannot create more particles than what fills ', &
                      'the allocated memory, sorry.'
                 write (*,*) 'num_local+(1+species(ss)%epp+species(ss)%hpp)', &
                      '*num_create=', num_local+(1+species(ss)%epp+ &
                      species(ss)%hpp)*num_create, ' nden=',nden, &
                      ' num_local=', num_local,' num_create=', num_create
                 stop
              end if
              
              do pp = 1, num_create
                 call random_number(rndp)
                 particles%species(num_local+1)      = ss
                 particles%coordinates(1,num_local+1)   = x0 + dxyz(1)*rndp(1)
                 particles%coordinates(2,num_local+1)   = y0 + dxyz(2)*rndp(2)
                 particles%coordinates(3,num_local+1)   = z0 + dxyz(3)*rndp(3)
                 particles%coordinates(4:6,num_local+1) = species(ss)%vn * &
                      particles%coordinates(1:3,num_local+1) / dsqrt(sum( &
                      particles%coordinates(1:3,num_local+1)**2.0d0)) + &
                      ui(ss) * rndp(4:6)/dsqrt(sum(rndp(4:6)**2.0d0))
                 particles%coordinates(1:3,num_local+1) = &
                      particles%coordinates(1:3,num_local+1) + &
                      particles%coordinates(4:6,num_local+1)*dt*rndp(7)

                 ! For each ion, create as many electrons as its charged state.
                 do ee = 1, species(ss)%epp
                    particles%species(num_local+1+ee) = &
                         species(ss)%productspecies
                    particles%coordinates(4:6,num_local+1+ee)   = &
                         particles%coordinates(4:6,num_local+1) - &
                         (ue(ss)+ui(ss))*rndp(4:6)/dsqrt(sum(rndp(4:6)**2.0d0))
                    particles%coordinates(1:3,num_local+1+ee)   = &
                         particles%coordinates(1:3,num_local+1) &
                         - particles%coordinates(4:6,num_local+1)*dt*rndp(7) &
                         + particles%coordinates(4:6,num_local+1+ee)*dt*rndp(7)
                 end do
                 do ee = 1, species(ss)%hpp
                    particles%species(num_local+1+species(ss)%epp+ee)       = 2
                    particles%coordinates(4:6,num_local+1+species(ss)%epp+ee) &
                         = particles%coordinates(4:6,num_local+1) - &
                         (uh(ss)+ui(ss))*rndp(4:6)/dsqrt(sum(rndp(4:6)**2.0d0))
                    particles%coordinates(1:3,num_local+1+species(ss)%epp+ee) &
                         = particles%coordinates(1:3,num_local+1) - &
                         particles%coordinates(4:6,num_local+1)*dt*rndp(7) + &
                         particles%coordinates(4:6,num_local+1+ &
                         species(ss)%epp+ee)*dt*rndp(7)
                 end do
                 num_local = num_local+1+species(ss)%epp+species(ss)%hpp
              end do

           end do
        end do
     end do
  end do

  return
end subroutine ParticlesProduce

!------------------------------------------------------------------------
