% picard_a3Eplot plots the E field from the picard program.
%
% Picard output files are assumed to be from the a3 version.
%
% HG 2021-09-30

fontname = 'utopia';

if exist('cometframe') ~= 1
  % Default to showing E fields in the comet frame of reference
  cometframe = logical(1);
end

run('inputpicarda3.m')
Nx=Nx_local*iprocs;
Ny=Ny_local*jprocs;
Nz=Nz_local*kprocs;
dxyz=[(xmax-xmin)/Nx (ymax-ymin)/Ny (zmax-zmin)/Nz];
xcorn = xmin + [0:Nx]*dxyz(1); x = 0.5*(xcorn(1:end-1)+xcorn(2:end));
ycorn = ymin + [0:Ny]*dxyz(2); y = 0.5*(ycorn(1:end-1)+ycorn(2:end));
zcorn = zmin + [0:Nz]*dxyz(3); z = 0.5*(zcorn(1:end-1)+zcorn(2:end));

sp = h5info('outp/parameters.h5');
clear species
for ii = 1:Nspecies
    species(ii) = h5read('outp/parameters.h5',['/' sp.Datasets(ii).Name]);
end

dd = h5info('outp/Efield.h5');

for kk = 1:length(dd.Groups)
  Ex = h5read('outp/Efield.h5',[dd.Groups(kk).Name '/Ex']);
  Ey = h5read('outp/Efield.h5',[dd.Groups(kk).Name '/Ey']);
  Ez = h5read('outp/Efield.h5',[dd.Groups(kk).Name '/Ez']);

  % Compute the solar wind E field, assuming the solar wind velocity is the
  % velocity of species 2.
  vsw = (species(1).mass*species(1).v0(1:3) + ...
         species(2).mass*species(2).v0(1:3))/ ...
        (species(1).mass+species(2).mass); % mean solar wind velocity,
                                           % assuming only two SW species
  B0 = [B0x B0y B0z];
  Esw = -cross(vsw,B0);

  % Eplus is added to the field to get us from the solar wind frame to the
  % comet frame if that is desired. Otherwise Eplus is zero.
  if cometframe
    Eplus = Esw;
    frametext = 'Comet frame';
  else
    Eplus = [0 0 0];
    frametext = 'SW frame';
  end

  figure(300+kk)
  clf
  fpos = get(gcf,'pos');
  set(gcf,'paperpositionmode','auto', ...
          'position',[fpos(1) fpos(2)+fpos(4)-420 1600 420])
  ha(1)=axes('position',[0.01 0.12 0.3 0.81]);
  ha(2)=axes('position',[0.35 0.12 0.3 0.81]);
  ha(3)=axes('position',[0.69 0.12 0.3 0.81]);

  [a iy0]=min(abs(ycorn-0));
  clear Exp Ezp
  Exp(:,:) = mean(Ex(:,[iy0-1:iy0],:),2)+Eplus(1);
  Ezp(:,:) = mean(Ez(:,[iy0-1:iy0],:),2)+Eplus(3);
  Emag=sqrt(Exp.^2 + Ezp.^2);
  pp(:,:) = Emag;pp=pp.';ppmax=ceil(max(max(pp)));
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  hs=surf(ha(1),xcorn,zcorn,pp-ppmax);
  clear pp
  set(ha(1),'fontname',fontname,'fontsize',14)
  view(ha(1),2)
  axis(ha(1),'square')
  axis(ha(1),[xmin xmax zmin zmax])
  shading(ha(1),'flat')
  xlabel(ha(1),'x','fontname',fontname,'fontsize',18)
  ylabel(ha(1),'z','fontname',fontname,'fontsize',18)
  hc=colorbar(ha(1));
  a=get(hc,'yticklabel');
  for ii=1:length(a)
    a{ii}=num2str(str2num(a{ii})+ppmax);
  end
  set(hc,'yticklabel',a)
  title(ha(1),['$\sqrt{E_{x}^{2}+E_{z}^{2}}$   ' frametext], ...
        'interpreter','latex','fontname',fontname,'fontsize',14)

  % add little arrows
  vv=round([10:10:240]*length(x)/240);
  xmat=x(vv).'*ones(size(z(vv)));
  zmat=ones(size(x(vv).'))*z(vv);
  maskx=logical(zeros(size(Exp)));maskx(vv,:)=logical(1);
  maskz=logical(zeros(size(Ezp)));maskz(:,vv)=logical(1);
  mask = logical(maskx.*maskz);

  hold(ha(1),'on')
  quiver(ha(1),xmat,zmat,reshape(Exp(mask),length(vv),length(vv)), ...
         reshape(Ezp(mask),length(vv),length(vv)),'w','autoscalefactor',2);
  hold(ha(1),'off')


  % a zoomed in plot
  x1=-180; %limits of plot axes
  x2=180;
  z1=-180;
  z2=180;
  Emagmax=max(max(Emag(x>=x1 & x<=x2, z>=z1 & z<=z2)));
  pp(:,:) = Emag;pp=pp.';ppmax=ceil(Emagmax);
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  hs=surf(ha(2),xcorn,zcorn,pp-ppmax);
  cax=caxis(ha(2));caxis(ha(2),[cax(1) Emagmax-ppmax])
  clear pp
  set(ha(2),'fontname',fontname,'fontsize',14)
  view(ha(2),2)
  axis(ha(2),'square')
  axis(ha(2),[x1 x2 z1 z2])
  shading(ha(2),'flat')
  xlabel(ha(2),'x','fontname',fontname,'fontsize',18)
  ylabel(ha(2),'z','fontname',fontname,'fontsize',18)
  hc=colorbar(ha(2));
  a=get(hc,'yticklabel');
  for ii=1:length(a)
    a{ii}=num2str(str2num(a{ii})+ppmax);
  end
  set(hc,'yticklabel',a)
  title(ha(2),['$\sqrt{E_{x}^{2}+E_{z}^{2}}$   ' frametext], ...
        'interpreter','latex','fontname',fontname,'fontsize',14)

  % add little arrows
  vv=round([96:4:145]*length(x)/240);
  xmat=x(vv).'*ones(size(z(vv)));
  zmat=ones(size(x(vv).'))*z(vv);
  maskx=logical(zeros(size(Exp)));maskx(vv,:)=logical(1);
  maskz=logical(zeros(size(Ezp)));maskz(:,vv)=logical(1);
  mask = logical(maskx.*maskz);

  hold(ha(2),'on')
  hq=quiver(ha(2),xmat,zmat,reshape(Exp(mask),length(vv),length(vv)), ...
            reshape(Ezp(mask),length(vv),length(vv)),'w', ...
            'autoscalefactor',3);
  hold(ha(2),'off')

  
  % a very zoomed in plot
  x1=-30; %limits of plot axes
  x2=30;
  z1=-30;
  z2=30;
  Emagmax=max(max(Emag(x>=x1 & x<=x2, z>=z1 & z<=z2)));
  pp(:,:) = Emag;pp=pp.';ppmax=ceil(Emagmax);
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  hs=surf(ha(3),xcorn,zcorn,pp-ppmax);
  cax=caxis(ha(3));caxis(ha(3),[cax(1) Emagmax-ppmax])
  clear pp
  set(ha(3),'fontname',fontname,'fontsize',14)
  view(ha(3),2)
  axis(ha(3),'square')
  axis(ha(3),[x1 x2 z1 z2])
  shading(ha(3),'flat')
  xlabel(ha(3),'x','fontname',fontname,'fontsize',18)
  ylabel(ha(3),'z','fontname',fontname,'fontsize',18)
  hc=colorbar(ha(3));
  a=get(hc,'yticklabel');
  for ii=1:length(a)
    a{ii}=num2str(str2num(a{ii})+ppmax);
  end
  set(hc,'yticklabel',a)
  title(ha(3),['$\sqrt{E_{x}^{2}+E_{z}^{2}}$   ' frametext], ...
        'interpreter','latex','fontname',fontname,'fontsize',14)

  % add little arrows
  vv=find((x>=x1 & x<=x2));
  xmat=x(vv).'*ones(size(z(vv)));
  zmat=ones(size(x(vv).'))*z(vv);
  maskx=logical(zeros(size(Exp)));maskx(vv,:)=logical(1);
  maskz=logical(zeros(size(Ezp)));maskz(:,vv)=logical(1);
  mask = logical(maskx.*maskz);

  hold(ha(3),'on')
  hq=quiver(ha(3),xmat,zmat,reshape(Exp(mask),length(vv),length(vv)), ...
            reshape(Ezp(mask),length(vv),length(vv)),'w', ...
            'autoscalefactor',1);
  hold(ha(3),'off')

  timestring=num2str(str2num(dd.Groups(kk).Name(6:end))*dt*1000);
  for ii=1:length(ha)
    aa=axis(ha(ii));
    cax=double(caxis(ha(ii)));
    text(aa(1)+0.05*diff(aa(1:2)), aa(3)+0.95*diff(aa(3:4)),cax(2)+1, ...
         ['$t=' timestring '$\,ms'],'interpreter','latex', ...
         'fontname',fontname,'fontsize',18,'parent',ha(ii))
  end
end




if 3==4
  h=get(0,'children');
  figlist=[];for ii=1:length(h);figlist=[figlist h(ii).Number];end
  [y, ind]=sort(figlist);
  ccc = pwd;
  cd ~
  for ii =1:length(ind)
    filename = ['fig' num2str(ii,'%0.2d') '.png'];
    print(h(ind(ii)),'-r600','-dpng',filename)
  end
  cd(ccc)
end
