% picard_a3plot plots outputs from the picard program.
%
% Picard output files are assumed to be from the a3 version.
%
% HG 2021-09-30

fontname = 'utopia';

if exist('cometframe') ~= 1
  % Default to showing E fields in the comet frame of reference
  cometframe = logical(1);
end

run('inputpicarda3.m')
Nx=Nx_local*iprocs;
Ny=Ny_local*jprocs;
Nz=Nz_local*kprocs;
dxyz=[(xmax-xmin)/Nx (ymax-ymin)/Ny (zmax-zmin)/Nz];
xcorn = xmin + [0:Nx]*dxyz(1); x = 0.5*(xcorn(1:end-1)+xcorn(2:end));
ycorn = ymin + [0:Ny]*dxyz(2); y = 0.5*(ycorn(1:end-1)+ycorn(2:end));
zcorn = zmin + [0:Nz]*dxyz(3); z = 0.5*(zcorn(1:end-1)+zcorn(2:end));

sp = h5info('outp/parameters.h5');
clear species
for ii = 1:Nspecies
    species(ii) = h5read('outp/parameters.h5',['/' sp.Datasets(ii).Name]);
end

dd = dir('outp/density*.h5');
clear Pstruct % P as in Picard
for ii = 1:length(dd)
    den=h5info(['outp/' dd(ii).name]);
    Pstruct(ii).density = h5read(['outp/' dd(ii).name], ...
                                 [den.Groups(end).Name '/density']);
end

clear pp
for ii = 1:Nspecies
  figure(20+ii)
  clf
  fpos = get(gcf,'pos');
  set(gcf,'paperpositionmode','auto', ...
          'position',[fpos(1) fpos(2)+fpos(4)-420 1331 420])
  subplot(1,3,1)
  [a ix0]=min(abs(xcorn-0));
  pp(:,:) = log10(mean(Pstruct(ii).density([ix0-1:ix0],:,:),1));pp=pp.';
  pp(isnan(pp) | isinf(pp)) = -15;
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(ycorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([ymin ymax zmin zmax])
  shading flat
  xlabel('y','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  cax=caxis;
  caxis([5 cax(2)])
  title(['n_{' num2str(ii) '}'],'fontname',fontname,'fontsize',14)
  
  subplot(1,3,2)
  [a iy0]=min(abs(ycorn-0));
  pp(:,:) = log10(mean(Pstruct(ii).density(:,[iy0-1:iy0],:),2));pp=pp.';
  pp(isnan(pp) | isinf(pp)) = -15;
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax zmin zmax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  cax=caxis;
  caxis([5 cax(2)])
  title(['n_{' num2str(ii) '}'],'fontname',fontname,'fontsize',14)

  subplot(1,3,3)
  [a iz0]=min(abs(zcorn-0));
  pp(:,:) = log10(mean(Pstruct(ii).density(:,:,[iz0-1:iz0]),3));pp=pp.';
  pp(isnan(pp) | isinf(pp)) = -15;
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,ycorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax ymin ymax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('y','fontname',fontname,'fontsize',18)
  colorbar
  cax=caxis;
  caxis([5 max(cax(2),5.01)])
  title(['n_{' num2str(ii) '}'],'fontname',fontname,'fontsize',14)
end

% load the E field
dd = h5info('outp/Efield.h5');
Ex = h5read('outp/Efield.h5',[dd.Groups(end).Name '/Ex']);
Ey = h5read('outp/Efield.h5',[dd.Groups(end).Name '/Ey']);
Ez = h5read('outp/Efield.h5',[dd.Groups(end).Name '/Ez']);
% load the B field
Bx = h5read('outp/Bfield.h5','/Bx');
By = h5read('outp/Bfield.h5','/By');
Bz = h5read('outp/Bfield.h5','/Bz');


% Compute the solar wind E field
vsw = (species(1).mass*species(1).v0(1:3) + ...
       species(2).mass*species(2).v0(1:3))/ ...
      (species(1).mass+species(2).mass); % mean solar wind velocity,
                                         % assuming only two SW species
Eswx = -(vsw(2)*Bz - vsw(3)*By);
Eswy = -(vsw(3)*Bx - vsw(1)*Bz);
Eswz = -(vsw(1)*By - vsw(2)*Bx);

% The -vxB field is added to the field to get us from the solar wind
% frame to the comet frame if that is desired. Here the label is set.
if cometframe
  frametext = 'Comet frame';
else
  frametext = 'SW frame';
end

figure(30)
clf
  fpos = get(gcf,'pos');
set(gcf,'paperpositionmode','auto', ...
        'position',[fpos(1) fpos(2)+fpos(4)-620 1017 620])
subplot(3,3,1)
[a ix0]=min(abs(xcorn-0));
pp(:,:) = mean(Ex([ix0-1:ix0],:,:),1);
if cometframe 
    pp = pp + squeeze(mean(Eswx([ix0-1:ix0],:,:),1));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(ycorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([ymin ymax zmin zmax])
shading flat
xlabel('y','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['E_{x} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,2)
[a iy0]=min(abs(ycorn-0));
pp(:,:) = mean(Ex(:,[iy0-1:iy0],:),2);
if cometframe 
    pp = pp + squeeze(mean(Eswx(:,[iy0-1:iy0],:),2));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax zmin zmax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['E_{x} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,3)
[a iz0]=min(abs(zcorn-0));
pp(:,:) = mean(Ex(:,:,[iz0-1:iz0]),3);
if cometframe 
    pp = pp + squeeze(mean(Eswx(:,:,[iz0-1:iz0]),3));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,ycorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax ymin ymax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('y','fontname',fontname,'fontsize',18)
colorbar
title(['E_{x} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,4)
[a ix0]=min(abs(xcorn-0));
pp(:,:) = mean(Ey([ix0-1:ix0],:,:),1);
if cometframe 
    pp = pp + squeeze(mean(Eswy([ix0-1:ix0],:,:),1));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(ycorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([ymin ymax zmin zmax])
shading flat
xlabel('y','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['E_{y} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,5)
[a iy0]=min(abs(ycorn-0));
pp(:,:) = mean(Ey(:,[iy0-1:iy0],:),2);
if cometframe 
    pp = pp + squeeze(mean(Eswy(:,[iy0-1:iy0],:),2));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax zmin zmax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['E_{y} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,6)
[a iz0]=min(abs(zcorn-0));
pp(:,:) = mean(Ey(:,:,[iz0-1:iz0]),3);
if cometframe
    pp = pp + squeeze(mean(Eswy(:,:,[iz0-1:iz0]),3));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,ycorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax ymin ymax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('y','fontname',fontname,'fontsize',18)
colorbar
title(['E_{y} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,7)
[a ix0]=min(abs(xcorn-0));
pp(:,:) = mean(Ez([ix0-1:ix0],:,:),1);
if cometframe 
    pp = pp + squeeze(mean(Eswz([ix0-1:ix0],:,:),1));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(ycorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([ymin ymax zmin zmax])
shading flat
xlabel('y','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['E_{z} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,8)
[a iy0]=min(abs(ycorn-0));
pp(:,:) = mean(Ez(:,[iy0-1:iy0],:),2);
if cometframe 
    pp = pp + squeeze(mean(Eswz(:,[iy0-1:iy0],:),2));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax zmin zmax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['E_{z} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,9)
[a iz0]=min(abs(zcorn-0));
pp(:,:) = mean(Ez(:,:,[iz0-1:iz0]),3);
if cometframe 
    pp = pp + squeeze(mean(Eswz(:,:,[iz0-1:iz0]),3));
end
pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,ycorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax ymin ymax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('y','fontname',fontname,'fontsize',18)
colorbar
title(['E_{z} ' frametext],'fontname',fontname,'fontsize',14)


figure(35)
clf
  fpos = get(gcf,'pos');
set(gcf,'paperpositionmode','auto', ...
        'position',[fpos(1) fpos(2)+fpos(4)-620 1017 620])
subplot(3,3,1)
[a ix0]=min(abs(xcorn-0));
pp(:,:) = mean(Bx([ix0-1:ix0],:,:),1);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(ycorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([ymin ymax zmin zmax])
shading flat
xlabel('y','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['B_{x} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,2)
[a iy0]=min(abs(ycorn-0));
pp(:,:) = mean(Bx(:,[iy0-1:iy0],:),2);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax zmin zmax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['B_{x} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,3)
[a iz0]=min(abs(zcorn-0));
pp(:,:) = mean(Bx(:,:,[iz0-1:iz0]),3);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,ycorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax ymin ymax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('y','fontname',fontname,'fontsize',18)
colorbar
title(['B_{x} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,4)
[a ix0]=min(abs(xcorn-0));
pp(:,:) = mean(By([ix0-1:ix0],:,:),1);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(ycorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([ymin ymax zmin zmax])
shading flat
xlabel('y','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['B_{y} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,5)
[a iy0]=min(abs(ycorn-0));
pp(:,:) = mean(By(:,[iy0-1:iy0],:),2);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax zmin zmax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['B_{y} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,6)
[a iz0]=min(abs(zcorn-0));
pp(:,:) = mean(By(:,:,[iz0-1:iz0]),3);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,ycorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax ymin ymax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('y','fontname',fontname,'fontsize',18)
colorbar
title(['B_{y} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,7)
[a ix0]=min(abs(xcorn-0));
pp(:,:) = mean(Bz([ix0-1:ix0],:,:),1);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(ycorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([ymin ymax zmin zmax])
shading flat
xlabel('y','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['B_{z} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,8)
[a iy0]=min(abs(ycorn-0));
pp(:,:) = mean(Bz(:,[iy0-1:iy0],:),2);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax zmin zmax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['B_{z} ' frametext],'fontname',fontname,'fontsize',14)

subplot(3,3,9)
[a iz0]=min(abs(zcorn-0));
pp(:,:) = mean(Bz(:,:,[iz0-1:iz0]),3);pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,ycorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax ymin ymax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('y','fontname',fontname,'fontsize',18)
colorbar
title(['B_{z} ' frametext],'fontname',fontname,'fontsize',14)


dd = h5info('outp/potential.h5');
U  = h5read('outp/potential.h5',[dd.Groups(end).Name '/potential']);


figure(40)
clf
fpos = get(gcf,'pos');
set(gcf,'paperpositionmode','auto', ...
        'position',[fpos(1) fpos(2)+fpos(4)-420 1331 420])
subplot(1,3,1)
[a ix0]=min(abs(xcorn-0));
% for the frame transformation
Uplus =-cumsum(squeeze(mean(Eswy([ix0-1:ix0],:,:),1)),1)*dxyz(2) - ...
       cumsum(squeeze(mean(Eswz([ix0-1:ix0],:,:),1)),2)*dxyz(3);

pp(:,:) = squeeze(mean(U([ix0-1:ix0],:,:),1))+Uplus;pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(ycorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([ymin ymax zmin zmax])
shading flat
xlabel('y','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['Potential, ' frametext],'fontname',fontname,'fontsize',14)

subplot(1,3,2)
[a iy0]=min(abs(ycorn-0));
% for the frame transformation
Uplus =-cumsum(squeeze(mean(Eswx(:,[iy0-1:iy0],:),2)),1)*dxyz(1) - ...
       cumsum(squeeze(mean(Eswz(:,[iy0-1:iy0],:),2)),2)*dxyz(3);
pp(:,:) = squeeze(mean(U(:,[iy0-1:iy0],:),2))+Uplus;pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,zcorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax zmin zmax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('z','fontname',fontname,'fontsize',18)
colorbar
title(['Potential, ' frametext],'fontname',fontname,'fontsize',14)

subplot(1,3,3)
[a iz0]=min(abs(zcorn-0));
% for the frame transformation
Uplus =-cumsum(squeeze(mean(Eswx(:,:,[iz0-1:iz0]),3)),1)*dxyz(1) - ...
       cumsum(squeeze(mean(Eswy(:,:,[iz0-1:iz0]),3)),2)*dxyz(2);
pp(:,:) = squeeze(mean(U(:,:,[iz0-1:iz0]),3))+Uplus;pp=pp.';
ss=size(pp);
pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
surf(xcorn,ycorn,pp)
clear pp
set(gca,'fontname',fontname,'fontsize',14)
view(2)
axis square
axis([xmin xmax ymin ymax])
shading flat
xlabel('x','fontname',fontname,'fontsize',18)
ylabel('y','fontname',fontname,'fontsize',18)
colorbar
title(['Potential, ' frametext],'fontname',fontname,'fontsize',14)


for ii = 1:Nspecies
  dd = dir(['outp/flux' num2str(ii,'%2.2i') '*.h5']);
  flu=h5info(['outp/' dd.name]);
  Fx = h5read(['outp/' dd.name],[flu.Groups(end).Name '/Fx']);
  Fy = h5read(['outp/' dd.name],[flu.Groups(end).Name '/Fy']);
  Fz = h5read(['outp/' dd.name],[flu.Groups(end).Name '/Fz']);

  figure(50+ii)
  clf
  fpos = get(gcf,'pos');
  set(gcf,'paperpositionmode','auto', ...
          'position',[fpos(1) fpos(2)+fpos(4)-620 1017 620])
  subplot(3,3,1)
  [a ix0]=min(abs(xcorn-0));
  pp(:,:) = mean(Fx([ix0-1:ix0],:,:),1);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(ycorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([ymin ymax zmin zmax])
  shading flat
  xlabel('y','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{x}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,2)
  [a iy0]=min(abs(ycorn-0));
  pp(:,:) = mean(Fx(:,[iy0-1:iy0],:),2);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax zmin zmax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{x}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,3)
  [a iz0]=min(abs(zcorn-0));
  pp(:,:) = mean(Fx(:,:,[iz0-1:iz0]),3);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,ycorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax ymin ymax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('y','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{x}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,4)
  [a ix0]=min(abs(xcorn-0));
  pp(:,:) = mean(Fy([ix0-1:ix0],:,:),1);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(ycorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([ymin ymax zmin zmax])
  shading flat
  xlabel('y','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{y}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,5)
  [a iy0]=min(abs(ycorn-0));
  pp(:,:) = mean(Fy(:,[iy0-1:iy0],:),2);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax zmin zmax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{y}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,6)
  [a iz0]=min(abs(zcorn-0));
  pp(:,:) = mean(Fy(:,:,[iz0-1:iz0]),3);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,ycorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax ymin ymax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('y','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{y}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,7)
  [a ix0]=min(abs(xcorn-0));
  pp(:,:) = mean(Fz([ix0-1:ix0],:,:),1);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(ycorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([ymin ymax zmin zmax])
  shading flat
  xlabel('y','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{z}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,8)
  [a iy0]=min(abs(ycorn-0));
  pp(:,:) = mean(Fz(:,[iy0-1:iy0],:),2);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,zcorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax zmin zmax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('z','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{z}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

  subplot(3,3,9)
  [a iz0]=min(abs(zcorn-0));
  pp(:,:) = mean(Fz(:,:,[iz0-1:iz0]),3);pp=pp.';
  ss=size(pp);
  pp=[[pp zeros(ss(1),1)];zeros(1,ss(2)+1)];
  surf(xcorn,ycorn,pp)
  clear pp
  set(gca,'fontname',fontname,'fontsize',14)
  view(2)
  axis square
  axis([xmin xmax ymin ymax])
  shading flat
  xlabel('x','fontname',fontname,'fontsize',18)
  ylabel('y','fontname',fontname,'fontsize',18)
  colorbar
  title(['\Gamma_{z}, species ' num2str(ii)],'fontname',fontname,'fontsize',14)

end


if 3==4
  h=get(0,'children');
  figlist=[];for ii=1:length(h);figlist=[figlist h(ii).Number];end
  [y, ind]=sort(figlist);
  ccc = pwd;
  cd ~
  for ii =1:length(ind)
    filename = ['fig' num2str(ii,'%0.2d') '.png'];
    print(h(ind(ii)),'-r600','-dpng',filename)
  end
  cd(ccc)
end
