% picard_a3probeplot plots data from the probes of the picard program
%
% HG 2021-10-20

fontname='utopia';

if exist('cometframe') ~= 1
    % Default to showing probe fields in the comet frame of reference
    cometframe = logical(1);
end

run('inputpicarda3.m')

sp = h5info('outp/parameters.h5');
clear species
for ii = 1:Nspecies
    species(ii) = h5read('outp/parameters.h5',['/' sp.Datasets(ii).Name]);
end

% Compute the solar wind E field, assuming the solar wind velocity is the
% velocity of species 2.
vsw = (species(1).mass*species(1).v0(1:3) + ...
       species(2).mass*species(2).v0(1:3))/ ...
      (species(1).mass+species(2).mass); % mean solar wind velocity,
                                         % assuming only two SW species
B0 = [B0x B0y B0z];
Esw = -cross(vsw,B0).';

% Eplus is added to the field to get us from the solar wind frame to the
% comet frame if that is desired. Otherwise Eplus is zero.
if cometframe
    Eplus = Esw;
    frametext = 'Comet frame';
else
    Eplus = [0 0 0].';
    frametext = 'SW frame';
end

clear probestruct
for ii= 1:Nprobes
    filename = ['outp/probes/Eprobe' num2str(ii,'%4.4i') '.h5'];
    Eprobe = h5read(filename,'/Efield');
    timestepsEprobe = double(h5read(filename,'/iterations'));
    probestruct(ii) = h5read(filename,'/probe');

    figure(10000+ii)
    clf
    set(gcf,'paperpositionmode','auto')
    plot(timestepsEprobe*dt,Eprobe+Eplus)
    set(gca,'fontname',fontname,'fontsize',14)
    legend('E_x','E_y','E_z')
    grid on
    title(['$x=' num2str(probestruct(ii).rc(1)) ...
           '$\,m, $y=' num2str(probestruct(ii).rc(2)) ...
           '$\,m, $z=' num2str(probestruct(ii).rc(3)) '$\,m  ' frametext], ...
          'interpreter','latex','fontname',fontname,'fontsize',18)
    
    filename = ['outp/probes/Pprobe' num2str(ii,'%4.4i') '.h5'];
    ppr = h5info(filename);
    if length(ppr.Groups)>0
        r = h5read(filename,[ppr.Groups(end).Name '/r']);
        v = h5read(filename,[ppr.Groups(end).Name '/v']);
        %spind: species index 
        spind = h5read(filename,[ppr.Groups(end).Name '/species']);
        clear Pprobe
        for jj=1:Nspecies
            Pprobe(jj).r=r(:,spind==jj);
            Pprobe(jj).v=v(:,spind==jj);
        end
        clear r v
        figure(20000+ii)
        clf
        set(gcf,'paperpositionmode','auto')
        for jj=1:length(Pprobe)
            if ~isempty(Pprobe(jj).r)
                subplot(length(Pprobe),4,(jj-1)*4+1)
                plot3(Pprobe(jj).r(1,:),Pprobe(jj).r(2,:), ...
                      Pprobe(jj).r(3,:),'k.')
               xlabel('$x$','interpreter','latex', ...
                       'fontname',fontname,'fontsize',18)
                ylabel('$y$','interpreter','latex', ...
                       'fontname',fontname,'fontsize',18)
                zlabel('$z$','interpreter','latex', ...
                       'fontname',fontname,'fontsize',18)
                title(['$x=' num2str(probestruct(ii).rc(1)) ...
                       '$\,m, $y=' num2str(probestruct(ii).rc(2)) ...
                       '$\,m, $z=' num2str(probestruct(ii).rc(3)) '$\,m'], ...
                      'interpreter','latex','fontname',fontname,'fontsize',14)
                axis equal
                subplot(length(Pprobe),4,(jj-1)*4+2)
                minv = min(Pprobe(jj).v(1,:));
                minbin = floor(minv*10^(-floor(log10(abs(minv))))) * ...
                         10^(floor(log10(abs(minv))));
                maxv = max(Pprobe(jj).v(1,:));
                maxbin = ceil(maxv*10^(-floor(log10(abs(maxv))))) * ...
                         10^(floor(log10(abs(maxv))));
                bins=minbin+(maxbin-minbin)*(0.5+[0:9])/10;
                hist(Pprobe(jj).v(1,:),bins)
                aa=axis;axis([minbin maxbin aa(3:4)])
                xlabel('$v_{x}$','interpreter','latex', ...
                       'fontname',fontname,'fontsize',18)
                subplot(length(Pprobe),4,(jj-1)*4+3)
                minv = min(Pprobe(jj).v(2,:));
                minbin = floor(minv*10^(-floor(log10(abs(minv))))) * ...
                         10^(floor(log10(abs(minv))));
                maxv = max(Pprobe(jj).v(2,:));
                maxbin = ceil(maxv*10^(-floor(log10(abs(maxv))))) * ...
                         10^(floor(log10(abs(maxv))));
                bins=minbin+(maxbin-minbin)*(0.5+[0:9])/10;
                hist(Pprobe(jj).v(2,:),bins)
                aa=axis;axis([minbin maxbin aa(3:4)])
                xlabel('$v_{y}$','interpreter','latex', ...
                       'fontname',fontname,'fontsize',18)
                title(['Species ' num2str(jj)], ...
                      'interpreter','latex','fontname',fontname,'fontsize',14)
                subplot(length(Pprobe),4,(jj-1)*4+4)
                minv = min(Pprobe(jj).v(3,:));
                minbin = floor(minv*10^(-floor(log10(abs(minv))))) * ...
                         10^(floor(log10(abs(minv))));
                maxv = max(Pprobe(jj).v(3,:));
                maxbin = ceil(maxv*10^(-floor(log10(abs(maxv))))) * ...
                         10^(floor(log10(abs(maxv))));
                bins=minbin+(maxbin-minbin)*(0.5+[0:9])/10;
                hist(Pprobe(jj).v(3,:),bins)
                aa=axis;axis([minbin maxbin aa(3:4)])
                xlabel('$v_{z}$','interpreter','latex', ...
                       'fontname',fontname,'fontsize',18)
            end
        end
    end
end

if 3==4
  h=get(0,'children');
  figlist=[];for ii=1:length(h);figlist=[figlist h(ii).Number];end
  [y, ind]=sort(figlist);
  ccc = pwd;
  cd ~
  for ii =1:length(ind)
    filename = ['fig' num2str(ii,'%0.2d') '.png'];
    print(h(ind(ii)),'-r600','-dpng',filename)
  end
  cd(ccc)
end
