% This is the input file for the run with a scale factor of 200 that was
% used in the manuscript we shall submit to A&A in October 2022. It takes
% quite some time to run, so to reproduce it you will most likely have to
% run a series of consecutive runs, changing the startfromdumpfile
% parameter after the first one.

%GENERAL PARAMETERS
iter_end = 100000;       % The iteration after which execution ends
startfromdumpfile='no';  % if 'yes' the distribution is read from the
                         % dumps directory
dump_period_dump=100;    % number of iterations between dumps one can start from
dump_period_particles=10000000; % number of iterations between particle dumps
dump_period_pprobes=10000;%number of iterations between particle probe dumps
dump_period_fields=100;   % number of iterations between field dumps
dump_start = 1;           % iteration at which dumping shall commence
write_period_screen=10;   % number of iterations between showing onscreen
                          % life signs.

Nspecies = 4;        % number of species included in the simulation
dt   =  3.0d-6;      % timestep
iprocs = 8;          % processes in x dimension, should be multiple of 2
jprocs = 8;          % processes in y dimension, should be multiple of 2
kprocs = 8;          % processes in z dimension, should be multiple of 2
Nx_local = 38;       % should be equal to INTEGER*8-2, e.g., 6, 30, 54, 78, 102
Ny_local = 38;       % should be equal to INTEGER*8-2, e.g., 6, 30, 54, 78, 102
Nz_local = 38;       % should be equal to INTEGER*8-2, e.g., 6, 30, 54, 78, 102

xmin = -1216.0d0;     % physical domain
ymin = -1216.0d0;     % physical domain
zmin = -1216.0d0;     % physical domain
xmax =  1216.0d0;     % physical domain
ymax =  1216.0d0;     % physical domain
zmax =  1216.0d0;     % physical domain

nucleusradius = 1.0e1;   % radius of the nucleus [m], 2.0d3/200.0d0
flatradius = 2.0e1;      % radius of flat density part [m]. Any value is
                         % allowed, but it should be 2*nucleusradius to
                         % cap the profile at its maximum value
Galandradius = 1.076e3;  % outer radius of Galand model (scaled up from run0007
                         % in proportion to xmax etc.)
fadeoutradius = 1.178e3; % radius at which the cometary ion density is zero
                         % (scaled up from run0007 in proportion to xmax etc.)

Bmodel = 1;                  % Magnetic field model
                             % 1: constant field given by B0x, B0y, B0z
                             % 2: B0x=B0z=0; By going from B1 for r<=r1
                             % to B2 for r>=r2. 
B0x = 0.0d-9;                % Constant magnetic field in the simulation region.
B0y = 9.923296463385918d-7; % 5nT is in the range observed (Goetz et al. 2017)
B0z = 0.0d-9;                % Scaled thus: 200.0d0*4.9616482316929589d-9
B1 = 1.0d-6;                 % Inner B for Bmodel=2
B2 = 4.0e-7;                 % Outer B for Bmodel=2
r1 = 600;                    % Inner transition point for Bmodel=2
r2 = 1000;                   % Outer transition point for Bmodel=2

Nprobes = 0;                % Number of probes defined below
%END

%SPECIES 1 - electrons. Species 1 must be electrons.
ppc = 64; % particles per cell, will be recomputed for SW electrons
%mass = 9.10938356d-31;     % mass [kg]
mass = 1.821876712d-29;     % mass [kg] me*20 = 1.821876712e-29 kg
charge = -1.602176634d-19; % charge [C]
upstreamdensity = 7.78d5;  % upstream (SW) density [m^-3]. 
upstreamkelvin = 7.21d4;   % solar wind temperature for this species
v0x =-430d3;               % solar wind velocity x component [m/s]
v0y = 0.0d3;               % solar wind velocity y component [m/s]
v0z = 0.0d3;               % solar wind velocity z component [m/s]
vn  = 0.0d0;               % neutral gas radial velocity component
Qn  = 0.0d0;               % neutral gas production rate [s^-1]
Ek  = 0.0d0;               % excess energy in photo-ionisation [eV]
nu_i = 0.0d0;              % ionisation frequency [s^-1]
cometion='no';             % If 'yes', a Galand model density profile is created
                           % It should be 'no' for electrons.
productspecies = 0;        % species number for the electrons that quasi-
                           % neutralise this species. This must be 0
                           % for electrons
%END

%SPECIES 2 - solar wind protons. Species 2 must be solar wind protons.
ppc = 64; % particles per cell
mass = 1.672621898d-27;    % mass [kg]
charge = 1.602176634d-19;  % charge [C]
upstreamdensity = 7.78d5;  % upstream (SW) density [m^-3]
upstreamkelvin = 3.85d4;   % solar wind temperature for this species
v0x =-430d3;               % solar wind velocity x component [m/s]
v0y = 0.0d3;               % solar wind velocity y component [m/s]
v0z = 0.0d3;               % solar wind velocity z component [m/s]
vn  = 0.0d0;               % neutral gas radial velocity component
Qn  = 0.0d0;               % neutral gas production rate [s^-1]
Ek  = 0.0d0;               % excess energy in photo-ionisation [eV]
nu_i = 0.0d0;              % ionisation frequency [s^-1]
cometion='no';             % If 'yes', a Galand model density profile is created
                           % It should be 'no' for electrons.
productspecies = 1;        % species number for the electrons that quasi-
                           % neutralise this species. This must be 0
                           % for electrons
%END

%SPECIES 3 - cometary H2O+. 
ppc = 4;                   % particles per cell relative the upstream
                           % electron (species 1) value
mass = 2.9914162400592d-26;% mass [kg]
charge = 1.602176634d-19;  % charge [C]
upstreamdensity = 0.0d0;   % upstream (SW) density [m^-3]
upstreamkelvin = 0.0d0;    % solar wind temperature for this species
v0x = 0.0d3;               % solar wind velocity x component [m/s]
v0y = 0.0d3;               % solar wind velocity y component [m/s]
v0z = 0.0d3;               % solar wind velocity z component [m/s]
vn  = 6.045d2;             % neutral gas radial velocity component
Qn  = 4.3730d23;           % neutral gas production rate
                           % (here, 8.746e25/200) [s^-1]
Ek  = 15.0d0;              % excess energy in photo-ionisation [eV]
nu_i = 3.68d-8;            % ionisation frequency (here, 3.31d-7 at 1 AU,
                           % low UV) [s^-1] 
cometion='yes';            % If 'yes', a Galand model density profile is created
                           % It should be 'no' for electrons.
productspecies = 4;        % Species number for the electron produced
                           % when ionising this species. This must be 0
                           % for electrons
%END

%SPECIES 4 - electrons. These shall be electrons created in ionisations
ppc = 4;                   % particles per cell relative the upstream
                           % electron (species 1) value
%mass = 9.10938356d-31;     % mass [kg]
mass = 1.821876712d-29;    % mass [kg] me*20 = 1.821876712e-29 kg
charge = -1.602176634d-19; % charge [C]
upstreamdensity = 0.0d0;   % will be recomputed for SW electrons [m^-3]
upstreamkelvin = 0.0d0;    % solar wind temperature for this species
v0x = 0.0d3;               % solar wind velocity x component [m/s]
v0y = 0.0d3;               % solar wind velocity y component [m/s]
v0z = 0.0d3;               % solar wind velocity z component [m/s]
vn  = 0.0d0;               % neutral gas radial velocity component
Qn  = 0.0d0;               % neutral gas production rate [s^-1]
Ek  = 0.0d0;               % excess energy in photo-ionisation [eV]
nu_i = 0.0d0;              % ionisation frequency [s^-1]
cometion='no';             % If 'yes', a Galand model density profile is created
                           % It should be 'no' for electrons.
productspecies = 0;        % Species number for the electron produced
                           % when ionising this species. This must be 0
                           % for electrons
%END

% Here follow the probe definitions. Particles are saved at intervals
% given by dump_period_pprobes set in the general parameters section. The
% electric field is saved every timestep. Try to put (xc,yc,zc) at the 
% centre of a grid cell. The program finds the nearest centre, but if you
% have (xc,yc,zc) at a cell boundary, it is not clear where the nearest
% centre is.

% In the run used in the manuscript Nprobes was set to zero above, which
% means that none of the probes below were used. This was done to avoid a
% problem with the probes which I encountered when implementing saving
% the output into hdf5 files.

%PROBE 1 as near the centre as possible
xc = 4.0d0;                % x coordinate of probe
yc = 4.0d0;                % y coordinate of probe
zc = 4.0d0;                % z coordinate of probe
rprobe = 4.0e0;            % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 2 further out, but still in the flat part
xc =12.0d0;                % x coordinate of probe
yc =12.0d0;                % y coordinate of probe
zc =12.0d0;                % z coordinate of probe
rprobe =12.0e0;            % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 3 opposite from probe 2
xc =-12.0d0;               % x coordinate of probe
yc =-12.0d0;               % y coordinate of probe
zc =-12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 4 same distance from the origin but in the +y direction
xc =-12.0d0;               % x coordinate of probe
yc = 12.0d0;               % y coordinate of probe
zc =-12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 5 same distance from the origin but in the -y direction
xc = 12.0d0;               % x coordinate of probe
yc =-12.0d0;               % y coordinate of probe
zc = 12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 6 and z
xc =-12.0d0;               % x coordinate of probe
yc =-12.0d0;               % y coordinate of probe
zc = 12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 7 and -z
xc = 12.0d0;               % x coordinate of probe
yc = 12.0d0;               % y coordinate of probe
zc =-12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 8 farther away
xc =100.0d0;               % x coordinate of probe
yc = 12.0d0;               % y coordinate of probe
zc = 12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 9 possible Rosetta position 
xc = 12.0d0;               % x coordinate of probe
yc =100.0d0;               % y coordinate of probe
zc = 12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 10 possible Rosetta position 
xc = 12.0d0;               % x coordinate of probe
yc = 12.0d0;               % y coordinate of probe
zc =100.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 11
xc =-100.0d0;              % x coordinate of probe
yc =  12.0d0;              % y coordinate of probe
zc =  12.0d0;              % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 12
xc =  12.0d0;              % x coordinate of probe
yc =-100.0d0;              % y coordinate of probe
zc =  12.0d0;              % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 13
xc =  12.0d0;              % x coordinate of probe
yc =  12.0d0;              % y coordinate of probe
zc =-100.0d0;              % z coordinate of probe
rprobe = 12.0e0;            % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 14 far on the nightside
xc =-500.0d0;              % x coordinate of probe
yc =  16.0d0;              % y coordinate of probe
zc =  16.0d0;              % z coordinate of probe
rprobe =16.0e0;            % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 15 completing the inside flatradius part (probes 2-7) +x
xc = 12.0d0;               % x coordinate of probe
yc =-12.0d0;               % y coordinate of probe
zc =-12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 16 and -x
xc =-12.0d0;               % x coordinate of probe
yc = 12.0d0;               % y coordinate of probe
zc = 12.0d0;               % z coordinate of probe
rprobe = 12.0e0;           % Probe radius for particle saves.
                           % If rprobe<=0, no particles are saved.
%END

%PROBE 17 for wave purposes, only E-field
xc =  -4.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END

%PROBE 18 for wave purposes, only E-field
xc =-100.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END

%PROBE 19 for wave purposes, only E-field
xc =-196.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END

%PROBE 20 for wave purposes, only E-field
xc =-292.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END

%PROBE 21 for wave purposes, only E-field
xc =-388.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END

%PROBE 22 for wave purposes, only E-field
xc =-484.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END

%PROBE 23 for wave purposes, only E-field
xc =-580.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END

%PROBE 24 for wave purposes, only E-field
xc =-676.0d0;               % x coordinate of probe
yc =   4.0d0;               % y coordinate of probe
zc = 100.0d0;               % z coordinate of probe
rprobe = -1.0e0;            % Probe radius for particle saves.
                            % If rprobe<=0, no particles are saved.
%END
